@@ -1,6 +1,7 @@
-"""This is the docstring for the example.py module.  Modules names should
-have short, all-lowercase names.  The module name may have underscores if
-this improves readability.
+"""Docstring for the example.py module.
+
+Modules names should have short, all-lowercase names.  The module name may
+have underscores if this improves readability.
 
 Every module should have a docstring at the very top of the file.  The
 module's docstring may extend over multiple lines.  If your docstring does
@@ -34,9 +35,8 @@ import matplotlib.pyplot as plt
 # numpy module itself, unabbreviated.
 
 
-def foo(var1, var2, long_var_name='hi'):
-    r"""A one-line summary that does not use variable names or the
-    function name.
+def foo(var1, var2, *args, long_var_name='hi', **kwargs):
+    r"""Summarize the function in one line.
 
     Several sentences providing an extended description. Refer to
     variables using back-ticks, e.g. `var`.
@@ -51,8 +51,12 @@ def foo(var1, var2, long_var_name='hi'):
         The type above can either refer to an actual Python type
         (e.g. ``int``), or describe the type of the variable in more
         detail, e.g. ``(N,) ndarray`` or ``array_like``.
+    *args : iterable
+        Other arguments.
     long_var_name : {'hi', 'ho'}, optional
         Choices in brackets, default first when optional.
+    **kwargs : dict
+        Keyword arguments.
 
     Returns
     -------
@@ -78,7 +82,7 @@ def foo(var1, var2, long_var_name='hi'):
 
     See Also
     --------
-    numpy.array : relationship (optional)
+    numpy.array : Relationship (optional).
     numpy.ndarray : Relationship (optional), which could be fairly long, in
                     which case the line wraps here.
     numpy.dot, numpy.linalg.norm, numpy.eye
@@ -112,12 +116,14 @@ def foo(var1, var2, long_var_name='hi'):
     use the function.
 
     >>> a = [1, 2, 3]
-    >>> print [x + 3 for x in a]
+    >>> print([x + 3 for x in a])
     [4, 5, 6]
-    >>> print "a\n\nb"
+    >>> print("a\nb")
     a
     b
-
     """
-
+    # After closing class docstring, there should be one blank line to
+    # separate following codes (according to PEP257).
+    # But for function, method and module, there should be no blank lines
+    # after closing the docstring.
     pass
