@@ -67,7 +67,8 @@ code book.
 import warnings
 import numpy as np
 from collections import deque
-from scipy._lib._util import _asarray_validated
+from scipy._lib._util import _asarray_validated, check_random_state,\
+    rng_integers
 from scipy.spatial.distance import cdist
 
 from . import _vq
@@ -314,7 +315,8 @@ def _kmeans(obs, guess, thresh=1e-5):
     return code_book, prev_avg_dists[1]
 
 
-def kmeans(obs, k_or_guess, iter=20, thresh=1e-5, check_finite=True):
+def kmeans(obs, k_or_guess, iter=20, thresh=1e-5, check_finite=True,
+           *, seed=None):
     """
     Performs k-means on a set of observation vectors forming k clusters.
 
@@ -361,6 +363,18 @@ def kmeans(obs, k_or_guess, iter=20, thresh=1e-5, check_finite=True):
         (crashes, non-termination) if the inputs do contain infinities or NaNs.
         Default: True
 
+    seed : {None, int, `numpy.random.Generator`,
+            `numpy.random.RandomState`}, optional
+
+        Seed for initializing the pseudo-random number generator.
+        If `seed` is None (or `numpy.random`), the `numpy.random.RandomState`
+        singleton is used.
+        If `seed` is an int, a new ``RandomState`` instance is used,
+        seeded with `seed`.
+        If `seed` is already a ``Generator`` or ``RandomState`` instance then
+        that instance is used.
+        The default is None.
+
     Returns
     -------
     codebook : ndarray
@@ -368,6 +382,9 @@ def kmeans(obs, k_or_guess, iter=20, thresh=1e-5, check_finite=True):
        codebook[i] is represented with the code i. The centroids
        and codes generated represent the lowest distortion seen,
        not necessarily the globally minimal distortion.
+       Note that the number of centroids is not necessarily the same as the
+       ``k_or_guess`` parameter, because centroids assigned to no observations
+       are removed during iterations.
 
     distortion : float
        The mean (non-squared) Euclidean distance between the observations
@@ -384,6 +401,13 @@ def kmeans(obs, k_or_guess, iter=20, thresh=1e-5, check_finite=True):
     whiten : must be called prior to passing an observation matrix
        to kmeans.
 
+    Notes
+    -----
+    For more functionalities or optimal performance, you can use
+    `sklearn.cluster.KMeans <https://scikit-learn.org/stable/modules/generated/sklearn.cluster.KMeans.html>`_.
+    `This <https://hdbscan.readthedocs.io/en/latest/performance_and_scalability.html#comparison-of-high-performance-implementations>`_
+    is a benchmark result of several implementations.
+
     Examples
     --------
     >>> from numpy import array
@@ -404,8 +428,6 @@ def kmeans(obs, k_or_guess, iter=20, thresh=1e-5, check_finite=True):
     (array([[ 2.3110306 ,  2.86287398],    # random
            [ 0.93218041,  1.24398691]]), 0.85684700941625547)
 
-    >>> from numpy import random
-    >>> random.seed((1000,2000))
     >>> codes = 3
     >>> kmeans(whitened,codes)
     (array([[ 2.3110306 ,  2.86287398],    # random
@@ -414,10 +436,11 @@ def kmeans(obs, k_or_guess, iter=20, thresh=1e-5, check_finite=True):
 
     >>> # Create 50 datapoints in two clusters a and b
     >>> pts = 50
-    >>> a = np.random.multivariate_normal([0, 0], [[4, 1], [1, 4]], size=pts)
-    >>> b = np.random.multivariate_normal([30, 10],
-    ...                                   [[10, 2], [2, 1]],
-    ...                                   size=pts)
+    >>> rng = np.random.default_rng()
+    >>> a = rng.multivariate_normal([0, 0], [[4, 1], [1, 4]], size=pts)
+    >>> b = rng.multivariate_normal([30, 10],
+    ...                             [[10, 2], [2, 1]],
+    ...                             size=pts)
     >>> features = np.concatenate((a, b))
     >>> # Whiten data
     >>> whitened = whiten(features)
@@ -427,6 +450,7 @@ def kmeans(obs, k_or_guess, iter=20, thresh=1e-5, check_finite=True):
     >>> plt.scatter(whitened[:, 0], whitened[:, 1])
     >>> plt.scatter(codebook[:, 0], codebook[:, 1], c='r')
     >>> plt.show()
+
     """
     obs = _asarray_validated(obs, check_finite=check_finite)
     if iter < 1:
@@ -447,11 +471,13 @@ def kmeans(obs, k_or_guess, iter=20, thresh=1e-5, check_finite=True):
     if k < 1:
         raise ValueError("Asked for %d clusters." % k)
 
+    rng = check_random_state(seed)
+
     # initialize best distance value to a large value
     best_dist = np.inf
     for i in range(iter):
         # the initial code book is randomly selected from observations
-        guess = _kpoints(obs, k)
+        guess = _kpoints(obs, k, rng)
         book, dist = _kmeans(obs, guess, thresh=thresh)
         if dist < best_dist:
             best_book = book
@@ -459,7 +485,7 @@ def kmeans(obs, k_or_guess, iter=20, thresh=1e-5, check_finite=True):
     return best_book, best_dist
 
 
-def _kpoints(data, k):
+def _kpoints(data, k, rng):
     """Pick k points at random in data (one row = one observation).
 
     Parameters
@@ -470,6 +496,8 @@ def _kpoints(data, k):
         row is one observation.
     k : int
         Number of samples to generate.
+    rng : `numpy.random.Generator` or `numpy.random.RandomState`
+        Random number generator.
 
     Returns
     -------
@@ -477,11 +505,11 @@ def _kpoints(data, k):
         A 'k' by 'N' containing the initial centroids
 
     """
-    idx = np.random.choice(data.shape[0], size=k, replace=False)
+    idx = rng.choice(data.shape[0], size=k, replace=False)
     return data[idx]
 
 
-def _krandinit(data, k):
+def _krandinit(data, k, rng):
     """Returns k samples of a random variable whose parameters depend on data.
 
     More precisely, it returns k observations sampled from a Gaussian random
@@ -495,6 +523,8 @@ def _krandinit(data, k):
         row is one observation.
     k : int
         Number of samples to generate.
+    rng : `numpy.random.Generator` or `numpy.random.RandomState`
+        Random number generator.
 
     Returns
     -------
@@ -506,12 +536,12 @@ def _krandinit(data, k):
 
     if data.ndim == 1:
         cov = np.cov(data)
-        x = np.random.randn(k)
+        x = rng.standard_normal(size=k)
         x *= np.sqrt(cov)
     elif data.shape[1] > data.shape[0]:
         # initialize when the covariance matrix is rank deficient
         _, s, vh = np.linalg.svd(data - mu, full_matrices=False)
-        x = np.random.randn(k, s.size)
+        x = rng.standard_normal(size=(k, s.size))
         sVh = s[:, None] * vh / np.sqrt(data.shape[0] - 1)
         x = x.dot(sVh)
     else:
@@ -519,14 +549,14 @@ def _krandinit(data, k):
 
         # k rows, d cols (one row = one obs)
         # Generate k sample of a random variable ~ Gaussian(mu, cov)
-        x = np.random.randn(k, mu.size)
+        x = rng.standard_normal(size=(k, mu.size))
         x = x.dot(np.linalg.cholesky(cov).T)
 
     x += mu
     return x
 
 
-def _kpp(data, k):
+def _kpp(data, k, rng):
     """ Picks k points in the data based on the kmeans++ method.
 
     Parameters
@@ -537,6 +567,8 @@ def _kpp(data, k):
         row is one observation.
     k : int
         Number of samples to generate.
+    rng : `numpy.random.Generator` or `numpy.random.RandomState`
+        Random number generator.
 
     Returns
     -------
@@ -555,13 +587,13 @@ def _kpp(data, k):
 
     for i in range(k):
         if i == 0:
-            init[i, :] = data[np.random.randint(data.shape[0])]
+            init[i, :] = data[rng_integers(rng, data.shape[0])]
 
         else:
             D2 = cdist(init[:i,:], data, metric='sqeuclidean').min(axis=0)
             probs = D2/D2.sum()
             cumprobs = probs.cumsum()
-            r = np.random.rand()
+            r = rng.uniform()
             init[i, :] = data[np.searchsorted(cumprobs, r)]
 
     return init
@@ -586,7 +618,7 @@ _valid_miss_meth = {'warn': _missing_warn, 'raise': _missing_raise}
 
 
 def kmeans2(data, k, iter=10, thresh=1e-5, minit='random',
-            missing='warn', check_finite=True):
+            missing='warn', check_finite=True, *, seed=None):
     """
     Classify a set of observations into k clusters using the k-means algorithm.
 
@@ -637,6 +669,17 @@ def kmeans2(data, k, iter=10, thresh=1e-5, minit='random',
         Disabling may give a performance gain, but may result in problems
         (crashes, non-termination) if the inputs do contain infinities or NaNs.
         Default: True
+    seed : {None, int, `numpy.random.Generator`,
+            `numpy.random.RandomState`}, optional
+
+        Seed for initializing the pseudo-random number generator.
+        If `seed` is None (or `numpy.random`), the `numpy.random.RandomState`
+        singleton is used.
+        If `seed` is an int, a new ``RandomState`` instance is used,
+        seeded with `seed`.
+        If `seed` is already a ``Generator`` or ``RandomState`` instance then
+        that instance is used.
+        The default is None.
 
     Returns
     -------
@@ -665,26 +708,26 @@ def kmeans2(data, k, iter=10, thresh=1e-5, minit='random',
     Create z, an array with shape (100, 2) containing a mixture of samples
     from three multivariate normal distributions.
 
-    >>> np.random.seed(12345678)
-    >>> a = np.random.multivariate_normal([0, 6], [[2, 1], [1, 1.5]], size=45)
-    >>> b = np.random.multivariate_normal([2, 0], [[1, -1], [-1, 3]], size=30)
-    >>> c = np.random.multivariate_normal([6, 4], [[5, 0], [0, 1.2]], size=25)
+    >>> rng = np.random.default_rng()
+    >>> a = rng.multivariate_normal([0, 6], [[2, 1], [1, 1.5]], size=45)
+    >>> b = rng.multivariate_normal([2, 0], [[1, -1], [-1, 3]], size=30)
+    >>> c = rng.multivariate_normal([6, 4], [[5, 0], [0, 1.2]], size=25)
     >>> z = np.concatenate((a, b, c))
-    >>> np.random.shuffle(z)
+    >>> rng.shuffle(z)
 
     Compute three clusters.
 
     >>> centroid, label = kmeans2(z, 3, minit='points')
     >>> centroid
-    array([[-0.35770296,  5.31342524],
-           [ 2.32210289, -0.50551972],
-           [ 6.17653859,  4.16719247]])
+    array([[ 2.22274463, -0.61666946],  # may vary
+           [ 0.54069047,  5.86541444],
+           [ 6.73846769,  4.01991898]])
 
     How many points are in each cluster?
 
     >>> counts = np.bincount(label)
     >>> counts
-    array([52, 27, 21])
+    array([29, 51, 20])  # may vary
 
     Plot the clusters.
 
@@ -741,7 +784,8 @@ def kmeans2(data, k, iter=10, thresh=1e-5, minit='random',
         except KeyError as e:
             raise ValueError("Unknown init method %r" % (minit,)) from e
         else:
-            code_book = init_meth(data, k)
+            rng = check_random_state(seed)
+            code_book = init_meth(data, k, rng)
 
     for i in range(iter):
         # Compute the nearest neighbor for each obs using the current code book
