@@ -231,7 +231,7 @@ py::array r2r_fftpack(const py::array &in, const py::object &axes_,
 
 template<typename T> py::array dct_internal(const py::array &in,
   const py::object &axes_, int type, int inorm, py::object &out_,
-  size_t nthreads)
+  size_t nthreads, bool ortho)
   {
   auto axes = makeaxes(in, axes_);
   auto dims(copy_shape(in));
@@ -244,7 +244,6 @@ template<typename T> py::array dct_internal(const py::array &in,
   py::gil_scoped_release release;
   T fct = (type==1) ? norm_fct<T>(inorm, dims, axes, 2, -1)
                     : norm_fct<T>(inorm, dims, axes, 2);
-  bool ortho = inorm == 1;
   pocketfft::dct(dims, s_in, s_out, axes, type, d_in, d_out, fct, ortho,
     nthreads);
   }
@@ -252,16 +251,20 @@ template<typename T> py::array dct_internal(const py::array &in,
   }
 
 py::array dct(const py::array &in, int type, const py::object &axes_,
-  int inorm, py::object &out_, size_t nthreads)
+  int inorm, py::object &out_, size_t nthreads, const py::object & ortho_obj)
   {
+  bool ortho=inorm==1;
+  if (!ortho_obj.is_none())
+    ortho=ortho_obj.cast<bool>();
+
   if ((type<1) || (type>4)) throw std::invalid_argument("invalid DCT type");
   DISPATCH(in, f64, f32, flong, dct_internal, (in, axes_, type, inorm, out_,
-    nthreads))
+    nthreads, ortho))
   }
 
 template<typename T> py::array dst_internal(const py::array &in,
   const py::object &axes_, int type, int inorm, py::object &out_,
-  size_t nthreads)
+  size_t nthreads, bool ortho)
   {
   auto axes = makeaxes(in, axes_);
   auto dims(copy_shape(in));
@@ -274,7 +277,6 @@ template<typename T> py::array dst_internal(const py::array &in,
   py::gil_scoped_release release;
   T fct = (type==1) ? norm_fct<T>(inorm, dims, axes, 2, 1)
                     : norm_fct<T>(inorm, dims, axes, 2);
-  bool ortho = inorm == 1;
   pocketfft::dst(dims, s_in, s_out, axes, type, d_in, d_out, fct, ortho,
     nthreads);
   }
@@ -282,11 +284,15 @@ template<typename T> py::array dst_internal(const py::array &in,
   }
 
 py::array dst(const py::array &in, int type, const py::object &axes_,
-  int inorm, py::object &out_, size_t nthreads)
+  int inorm, py::object &out_, size_t nthreads, const py::object &ortho_obj)
   {
+  bool ortho=inorm==1;
+  if (!ortho_obj.is_none())
+    ortho=ortho_obj.cast<bool>();
+
   if ((type<1) || (type>4)) throw std::invalid_argument("invalid DST type");
   DISPATCH(in, f64, f32, flong, dst_internal, (in, axes_, type, inorm,
-    out_, nthreads))
+    out_, nthreads, ortho))
   }
 
 template<typename T> py::array c2r_internal(const py::array &in,
@@ -374,11 +380,13 @@ py::array genuine_hartley(const py::array &in, const py::object &axes_,
   }
 
 // Export good_size in raw C-API to reduce overhead (~4x faster)
-PyObject * good_size(PyObject * /*self*/, PyObject * args)
+PyObject * good_size(PyObject * /*self*/, PyObject * args, PyObject * kwargs)
   {
   Py_ssize_t n_ = -1;
   int real = false;
-  if (!PyArg_ParseTuple(args, "n|p:good_size", &n_, &real))
+  static const char * keywords[] = {"target", "real", nullptr};
+  if (!PyArg_ParseTupleAndKeywords(args, kwargs, "n|p:good_size",
+                                   (char **) keywords, &n_, &real))
     return nullptr;
 
   if (n_<0)
@@ -620,7 +628,7 @@ axes : list of integers
 inorm : int
     Normalization type
       0 : no normalization
-      1 : make transform orthogonal and divide by sqrt(N)
+      1 : divide by sqrt(N)
       2 : divide by N
     where N is the product of n_i for every transformed axis i.
     n_i is 2*(<axis_length>-1 for type 1 and 2*<axis length>
@@ -638,6 +646,8 @@ out : numpy.ndarray (same shape and data type as `a`)
 nthreads : int
     Number of threads to use. If 0, use the system default (typically governed
     by the `OMP_NUM_THREADS` environment variable).
+ortho: bool
+    Orthogonalize transform (defaults to ``inorm==1``)
 
 Returns
 -------
@@ -659,7 +669,7 @@ axes : list of integers
 inorm : int
     Normalization type
       0 : no normalization
-      1 : make transform orthogonal and divide by sqrt(N)
+      1 : divide by sqrt(N)
       2 : divide by N
     where N is the product of n_i for every transformed axis i.
     n_i is 2*(<axis_length>+1 for type 1 and 2*<axis length>
@@ -676,6 +686,8 @@ out : numpy.ndarray (same shape and data type as `a`)
 nthreads : int
     Number of threads to use. If 0, use the system default (typically governed
     by the `OMP_NUM_THREADS` environment variable).
+ortho: bool
+    Orthogonalize transform (defaults to ``inorm==1``)
 
 Returns
 -------
@@ -687,7 +699,7 @@ const char * good_size_DS = R"""(Returns a good length to pad an FFT to.
 
 Parameters
 ----------
-n : int
+target : int
     Minimum transform length
 real : bool, optional
     True if either input or output of FFT should be fully real.
@@ -719,11 +731,12 @@ PYBIND11_MODULE(pypocketfft, m)
   m.def("genuine_hartley", genuine_hartley, genuine_hartley_DS, "a"_a,
     "axes"_a=None, "inorm"_a=0, "out"_a=None, "nthreads"_a=1);
   m.def("dct", dct, dct_DS, "a"_a, "type"_a, "axes"_a=None, "inorm"_a=0,
-    "out"_a=None, "nthreads"_a=1);
+    "out"_a=None, "nthreads"_a=1, "ortho"_a=None);
   m.def("dst", dst, dst_DS, "a"_a, "type"_a, "axes"_a=None, "inorm"_a=0,
-    "out"_a=None, "nthreads"_a=1);
+    "out"_a=None, "nthreads"_a=1, "ortho"_a=None);
 
   static PyMethodDef good_size_meth[] =
-    {{"good_size", good_size, METH_VARARGS, good_size_DS}, {0}};
+    {{"good_size", (PyCFunction)good_size,
+      METH_VARARGS | METH_KEYWORDS, good_size_DS}, {0}};
   PyModule_AddFunctions(m.ptr(), good_size_meth);
   }
