@@ -6,7 +6,7 @@ import functools
 
 import numpy as np
 
-from scipy._lib._util import MapWrapper
+from scipy._lib._util import MapWrapper, _FunctionWrapper
 
 
 class LRUDict(collections.OrderedDict):
@@ -15,7 +15,7 @@ class LRUDict(collections.OrderedDict):
 
     def __setitem__(self, key, value):
         existing_key = (key in self)
-        super(LRUDict, self).__setitem__(key, value)
+        super().__setitem__(key, value)
         if existing_key:
             self.move_to_end(key)
         elif len(self) > self.__max_size:
@@ -26,7 +26,7 @@ class LRUDict(collections.OrderedDict):
         raise NotImplementedError()
 
 
-class SemiInfiniteFunc(object):
+class SemiInfiniteFunc:
     """
     Argument transform from (start, +-oo) to (0, 1)
     """
@@ -54,7 +54,7 @@ class SemiInfiniteFunc(object):
             return self._sgn * (f / t) / t
 
 
-class DoubleInfiniteFunc(object):
+class DoubleInfiniteFunc:
     """
     Argument transform from (-oo, oo) to (-1, 1)
     """
@@ -91,7 +91,7 @@ def _get_sizeof(obj):
         return 64
 
 
-class _Bunch(object):
+class _Bunch:
     def __init__(self, **kwargs):
         self.__keys = kwargs.keys()
         self.__dict__.update(**kwargs)
@@ -102,7 +102,8 @@ class _Bunch(object):
 
 
 def quad_vec(f, a, b, epsabs=1e-200, epsrel=1e-8, norm='2', cache_size=100e6, limit=10000,
-             workers=1, points=None, quadrature=None, full_output=False):
+             workers=1, points=None, quadrature=None, full_output=False,
+             *, args=()):
     r"""Adaptive integration of a vector-valued function.
 
     Parameters
@@ -140,6 +141,10 @@ def quad_vec(f, a, b, epsabs=1e-200, epsrel=1e-8, norm='2', cache_size=100e6, li
         Default: 'gk21' for finite intervals and 'gk15' for (semi-)infinite
     full_output : bool, optional
         Return an additional ``info`` dictionary.
+    args : tuple, optional
+        Extra arguments to pass to function, if any.
+
+        .. versionadded:: 1.8.0
 
     Returns
     -------
@@ -215,6 +220,13 @@ def quad_vec(f, a, b, epsabs=1e-200, epsrel=1e-8, norm='2', cache_size=100e6, li
     a = float(a)
     b = float(b)
 
+    if args:
+        if not isinstance(args, tuple):
+            args = (args,)
+
+        # create a wrapped function to allow the use of map and Pool.map
+        f = _FunctionWrapper(f, args)
+
     # Use simple transformations to deal with integrals over infinite
     # intervals.
     kwargs = dict(epsabs=epsabs,
@@ -267,7 +279,6 @@ def quad_vec(f, a, b, epsabs=1e-200, epsrel=1e-8, norm='2', cache_size=100e6, li
     else:
         norm_func = norm_funcs[norm]
 
-
     parallel_count = 128
     min_intervals = 2
 
