@@ -8,20 +8,18 @@
 # Modified: Ilhan Polat <ilhanpolat@gmail.com>
 # September 13, 2016
 
-from __future__ import division, print_function, absolute_import
-
 import warnings
 import numpy as np
 from numpy.linalg import inv, LinAlgError, norm, cond, svd
 
-from .basic import solve, solve_triangular, matrix_balance
+from ._basic import solve, solve_triangular, matrix_balance
 from .lapack import get_lapack_funcs
-from .decomp_schur import schur
-from .decomp_lu import lu
-from .decomp_qr import qr
+from ._decomp_schur import schur
+from ._decomp_lu import lu
+from ._decomp_qr import qr
 from ._decomp_qz import ordqz
-from .decomp import _asarray_validated
-from .special_matrices import kron, block_diag
+from ._decomp import _asarray_validated
+from ._special_matrices import kron, block_diag
 
 __all__ = ['solve_sylvester',
            'solve_continuous_lyapunov', 'solve_discrete_lyapunov',
@@ -55,11 +53,11 @@ def solve_sylvester(a, b, q):
     Notes
     -----
     Computes a solution to the Sylvester matrix equation via the Bartels-
-    Stewart algorithm.  The A and B matrices first undergo Schur
-    decompositions.  The resulting matrices are used to construct an
+    Stewart algorithm. The A and B matrices first undergo Schur
+    decompositions. The resulting matrices are used to construct an
     alternative Sylvester equation (``RY + YS^T = F``) where the R and S
     matrices are in quasi-triangular form (or, when R, S or F are complex,
-    triangular form).  The simplified equation is then solved using
+    triangular form). The simplified equation is then solved using
     ``*TRSYL`` from LAPACK directly.
 
     .. versionadded:: 0.11.0
@@ -82,10 +80,10 @@ def solve_sylvester(a, b, q):
 
     """
 
-    # Compute the Schur decomp form of a
+    # Compute the Schur decomposition form of a
     r, u = schur(a, output='real')
 
-    # Compute the Schur decomp of b
+    # Compute the Schur decomposition of b
     s, v = schur(b.conj().transpose(), output='real')
 
     # Construct f = u'*q*v
@@ -172,7 +170,7 @@ def solve_continuous_lyapunov(a, q):
     if a.shape != q.shape:
         raise ValueError("Matrix a and q should have the same shape.")
 
-    # Compute the Schur decomp form of a
+    # Compute the Schur decomposition form of a
     r, u = schur(a, output='real')
 
     # Construct f = u'*q*u
@@ -266,7 +264,7 @@ def solve_discrete_lyapunov(a, q, method=None):
     and ``bilinear`` otherwise.
 
     Method *direct* uses a direct analytical solution to the discrete Lyapunov
-    equation. The algorithm is given in, for example, [1]_. However it requires
+    equation. The algorithm is given in, for example, [1]_. However, it requires
     the linear solution of a system with dimension :math:`M^2` so that
     performance degrades rapidly for even moderately sized matrices.
 
@@ -351,7 +349,7 @@ def solve_continuous_are(a, b, q, r, e=None, s=None, balanced=True):
 
     is solved. When omitted, ``e`` is assumed to be the identity and ``s``
     is assumed to be the zero matrix with sizes compatible with ``a`` and
-    ``b`` respectively.
+    ``b``, respectively.
 
     Parameters
     ----------
@@ -400,7 +398,7 @@ def solve_continuous_are(a, b, q, r, e=None, s=None, balanced=True):
     In this algorithm, the fail conditions are linked to the symmetry
     of the product :math:`U_2 U_1^{-1}` and condition number of
     :math:`U_1`. Here, :math:`U` is the 2m-by-m matrix that holds the
-    eigenvectors spanning the stable subspace with 2m rows and partitioned
+    eigenvectors spanning the stable subspace with 2-m rows and partitioned
     into two m-row matrices. See [1]_ and [2]_ for more details.
 
     In order to improve the QZ decomposition accuracy, the pencil goes
@@ -414,7 +412,7 @@ def solve_continuous_are(a, b, q, r, e=None, s=None, balanced=True):
     ----------
     .. [1]  P. van Dooren , "A Generalized Eigenvalue Approach For Solving
        Riccati Equations.", SIAM Journal on Scientific and Statistical
-       Computing, Vol.2(2), DOI: 10.1137/0902010
+       Computing, Vol.2(2), :doi:`10.1137/0902010`
 
     .. [2] A.J. Laub, "A Schur Method for Solving Algebraic Riccati
        Equations.", Massachusetts Institute of Technology. Laboratory for
@@ -422,7 +420,7 @@ def solve_continuous_are(a, b, q, r, e=None, s=None, balanced=True):
        http://hdl.handle.net/1721.1/1301
 
     .. [3] P. Benner, "Symplectic Balancing of Hamiltonian Matrices", 2001,
-       SIAM J. Sci. Comput., 2001, Vol.22(5), DOI: 10.1137/S1064827500367993
+       SIAM J. Sci. Comput., 2001, Vol.22(5), :doi:`10.1137/S1064827500367993`
 
     Examples
     --------
@@ -603,7 +601,7 @@ def solve_discrete_are(a, b, q, r, e=None, s=None, balanced=True):
     In this algorithm, the fail conditions are linked to the symmetry
     of the product :math:`U_2 U_1^{-1}` and condition number of
     :math:`U_1`. Here, :math:`U` is the 2m-by-m matrix that holds the
-    eigenvectors spanning the stable subspace with 2m rows and partitioned
+    eigenvectors spanning the stable subspace with 2-m rows and partitioned
     into two m-row matrices. See [1]_ and [2]_ for more details.
 
     In order to improve the QZ decomposition accuracy, the pencil goes
@@ -619,7 +617,7 @@ def solve_discrete_are(a, b, q, r, e=None, s=None, balanced=True):
     ----------
     .. [1]  P. van Dooren , "A Generalized Eigenvalue Approach For Solving
        Riccati Equations.", SIAM Journal on Scientific and Statistical
-       Computing, Vol.2(2), DOI: 10.1137/0902010
+       Computing, Vol.2(2), :doi:`10.1137/0902010`
 
     .. [2] A.J. Laub, "A Schur Method for Solving Algebraic Riccati
        Equations.", Massachusetts Institute of Technology. Laboratory for
@@ -627,7 +625,7 @@ def solve_discrete_are(a, b, q, r, e=None, s=None, balanced=True):
        http://hdl.handle.net/1721.1/1301
 
     .. [3] P. Benner, "Symplectic Balancing of Hamiltonian Matrices", 2001,
-       SIAM J. Sci. Comput., 2001, Vol.22(5), DOI: 10.1137/S1064827500367993
+       SIAM J. Sci. Comput., 2001, Vol.22(5), :doi:`10.1137/S1064827500367993`
 
     Examples
     --------
@@ -744,12 +742,12 @@ def _are_validate_args(a, b, q, r, e, s, eq_type='care'):
 
     Essentially, it performs:
 
-        - a check whether the input is free of NaN and Infs.
+        - a check whether the input is free of NaN and Infs
         - a pass for the data through ``numpy.atleast_2d()``
-        - squareness check of the relevant arrays,
-        - shape consistency check of the arrays,
-        - singularity check of the relevant arrays,
-        - symmetricity check of the relevant matrices,
+        - squareness check of the relevant arrays
+        - shape consistency check of the arrays
+        - singularity check of the relevant arrays
+        - symmetricity check of the relevant matrices
         - a check whether the regular or the generalized version is asked.
 
     This function is used by ``solve_continuous_are`` and
