@@ -22,7 +22,7 @@ def get_mat(n):
     return data
 
 
-class TestTri(object):
+class TestTri:
     def test_basic(self):
         assert_equal(tri(4), array([[1, 0, 0, 0],
                                     [1, 1, 0, 0],
@@ -62,7 +62,7 @@ class TestTri(object):
                                              [1, 1, 0]]))
 
 
-class TestTril(object):
+class TestTril:
     def test_basic(self):
         a = (100*get_mat(5)).astype('l')
         b = a.copy()
@@ -85,7 +85,7 @@ class TestTril(object):
         assert_equal(tril(a, k=-2), b)
 
 
-class TestTriu(object):
+class TestTriu:
     def test_basic(self):
         a = (100*get_mat(5)).astype('l')
         b = a.copy()
@@ -108,7 +108,7 @@ class TestTriu(object):
         assert_equal(triu(a, k=-2), b)
 
 
-class TestToeplitz(object):
+class TestToeplitz:
 
     def test_basic(self):
         y = toeplitz([1, 2, 3])
@@ -155,7 +155,7 @@ class TestToeplitz(object):
         assert_array_equal(t, [[1, 2, 3]])
 
 
-class TestHankel(object):
+class TestHankel:
     def test_basic(self):
         y = hankel([1, 2, 3])
         assert_array_equal(y, [[1, 2, 3], [2, 3, 0], [3, 0, 0]])
@@ -163,13 +163,13 @@ class TestHankel(object):
         assert_array_equal(y, [[1, 2, 3], [2, 3, 4], [3, 4, 5]])
 
 
-class TestCirculant(object):
+class TestCirculant:
     def test_basic(self):
         y = circulant([1, 2, 3])
         assert_array_equal(y, [[1, 3, 2], [2, 1, 3], [3, 2, 1]])
 
 
-class TestHadamard(object):
+class TestHadamard:
 
     def test_basic(self):
 
@@ -189,7 +189,7 @@ class TestHadamard(object):
         assert_raises(ValueError, hadamard, 5)
 
 
-class TestLeslie(object):
+class TestLeslie:
 
     def test_bad_shapes(self):
         assert_raises(ValueError, leslie, [[1, 1], [2, 2]], [3, 4, 5])
@@ -205,7 +205,7 @@ class TestLeslie(object):
         assert_array_equal(a, expected)
 
 
-class TestCompanion(object):
+class TestCompanion:
 
     def test_bad_shapes(self):
         assert_raises(ValueError, companion, [[1, 1], [2, 2]])
@@ -313,7 +313,7 @@ class TestKron:
         assert_array_equal(a, expected)
 
 
-class TestHelmert(object):
+class TestHelmert:
 
     def test_orthogonality(self):
         for n in range(1, 7):
@@ -332,7 +332,7 @@ class TestHelmert(object):
                 assert_allclose(U.T.dot(U), np.eye(n-1), atol=1e-12)
 
 
-class TestHilbert(object):
+class TestHilbert:
 
     def test_basic(self):
         h3 = array([[1.0, 1/2., 1/3.],
@@ -346,7 +346,7 @@ class TestHilbert(object):
         assert_equal(h0.shape, (0, 0))
 
 
-class TestInvHilbert(object):
+class TestInvHilbert:
 
     def test_basic(self):
         invh1 = array([[1]])
@@ -514,7 +514,7 @@ class TestInvHilbert(object):
             assert_allclose(a.dot(b), eye(n), atol=1e-15*c, rtol=1e-15*c)
 
 
-class TestPascal(object):
+class TestPascal:
 
     cases = [
         (1, array([[1]]), array([[1]])),
