@@ -1,7 +1,7 @@
 /* Anti-Copyright
  *
- * I hereby release this code into the PUBLIC DOMAIN AS IS.  There is no
- * support, warranty, or guarantee.  I will gladly accept comments, bug 
+ * I hereby release this code into the PUBLIC DOMAIN AS IS. There is no
+ * support, warranty, or guarantee. I will gladly accept comments, bug
  * reports, and patches, however.
  *
  * Robert Kern
@@ -9,42 +9,43 @@
  *
  */
 
+#define PY_SSIZE_T_CLEAN
 #include "odrpack.h"
 
 
-void F_FUNC(dodrc,DODRC)(void (*fcn)(int *n, int *m, int *np, int *nq, int *ldn, int *ldm, 
-            int *ldnp, double *beta, double *xplusd, int *ifixb, int *ifixx, 
-            int *ldifx, int *ideval, double *f, double *fjacb, double *fjacd, 
-            int *istop), 
-           int *n, int *m, int *np, int *nq, double *beta, double *y, int *ldy,
-           double *x, int *ldx, double *we, int *ldwe, int *ld2we, double *wd,
-           int *ldwd, int *ld2wd, int *ifixb, int *ifixx, int *ldifx, int *job,
-           int *ndigit, double *taufac, double *sstol, double *partol, 
-           int *maxit, int *iprint, int *lunerr, int *lunrpt, double *stpb,
-           double *stpd, int *ldstpd, double *sclb, double *scld, int *ldscld,
-           double *work, int *lwork, int *iwork, int *liwork, int *info);
-void F_FUNC(dwinf,DWINF)(int *n, int *m, int *np, int *nq, int *ldwe, int *ld2we, int *isodr,
-        int *delta, int *eps, int *xplus, int *fn, int *sd, int *vcv, int *rvar,
-        int *wss, int *wssde, int *wssep, int *rcond, int *eta, int *olmav, 
-        int *tau, int *alpha, int *actrs, int *pnorm, int *rnors, int *prers,
-        int *partl, int *sstol, int *taufc, int *apsma, int *betao, int *betac,
-        int *betas, int *betan, int *s, int *ss, int *ssf, int *qraux, int *u,
-        int *fs, int *fjacb, int *we1, int *diff, int *delts, int *deltn, 
-        int *t, int *tt, int *omega, int *fjacd, int *wrk1, int *wrk2, 
-        int *wrk3, int *wrk4, int *wrk5, int *wrk6, int *wrk7, int *lwkmn);
-void F_FUNC(dluno,DLUNO)(int *lun, char *fn, int fnlen);
-void F_FUNC(dlunc,DLUNC)(int *lun);
+void F_FUNC(dodrc,DODRC)(void (*fcn)(F_INT *n, F_INT *m, F_INT *np, F_INT *nq, F_INT *ldn, F_INT *ldm,
+            F_INT *ldnp, double *beta, double *xplusd, F_INT *ifixb, F_INT *ifixx,
+            F_INT *ldifx, F_INT *ideval, double *f, double *fjacb, double *fjacd,
+            F_INT *istop),
+           F_INT *n, F_INT *m, F_INT *np, F_INT *nq, double *beta, double *y, F_INT *ldy,
+           double *x, F_INT *ldx, double *we, F_INT *ldwe, F_INT *ld2we, double *wd,
+           F_INT *ldwd, F_INT *ld2wd, F_INT *ifixb, F_INT *ifixx, F_INT *ldifx, F_INT *job,
+           F_INT *ndigit, double *taufac, double *sstol, double *partol,
+           F_INT *maxit, F_INT *iprF_INT, F_INT *lunerr, F_INT *lunrpt, double *stpb,
+           double *stpd, F_INT *ldstpd, double *sclb, double *scld, F_INT *ldscld,
+           double *work, F_INT *lwork, F_INT *iwork, F_INT *liwork, F_INT *info);
+void F_FUNC(dwinf,DWINF)(F_INT *n, F_INT *m, F_INT *np, F_INT *nq, F_INT *ldwe, F_INT *ld2we, F_INT *isodr,
+        F_INT *delta, F_INT *eps, F_INT *xplus, F_INT *fn, F_INT *sd, F_INT *vcv, F_INT *rvar,
+        F_INT *wss, F_INT *wssde, F_INT *wssep, F_INT *rcond, F_INT *eta, F_INT *olmav,
+        F_INT *tau, F_INT *alpha, F_INT *actrs, F_INT *pnorm, F_INT *rnors, F_INT *prers,
+        F_INT *partl, F_INT *sstol, F_INT *taufc, F_INT *apsma, F_INT *betao, F_INT *betac,
+        F_INT *betas, F_INT *betan, F_INT *s, F_INT *ss, F_INT *ssf, F_INT *qraux, F_INT *u,
+        F_INT *fs, F_INT *fjacb, F_INT *we1, F_INT *diff, F_INT *delts, F_INT *deltn,
+        F_INT *t, F_INT *tt, F_INT *omega, F_INT *fjacd, F_INT *wrk1, F_INT *wrk2,
+        F_INT *wrk3, F_INT *wrk4, F_INT *wrk5, F_INT *wrk6, F_INT *wrk7, F_INT *lwkmn);
+void F_FUNC(dluno,DLUNO)(F_INT *lun, char *fn, int fnlen);
+void F_FUNC(dlunc,DLUNC)(F_INT *lun);
 
 
 
 /* callback to pass to DODRC; calls the Python function in the global structure |odr_global| */
-void fcn_callback(int *n, int *m, int *np, int *nq, int *ldn, int *ldm,
-                  int *ldnp, double *beta, double *xplusd, int *ifixb,
-                  int *ifixx, int *ldfix, int *ideval, double *f,
-                  double *fjacb, double *fjacd, int *istop)
+void fcn_callback(F_INT *n, F_INT *m, F_INT *np, F_INT *nq, F_INT *ldn, F_INT *ldm,
+                  F_INT *ldnp, double *beta, double *xplusd, F_INT *ifixb,
+                  F_INT *ifixx, F_INT *ldfix, F_INT *ideval, double *f,
+                  double *fjacb, double *fjacd, F_INT *istop)
 {
   PyObject *arg01, *arglist;
-  PyObject *result;
+  PyObject *result = NULL;
   PyArrayObject *result_array = NULL;
   PyArrayObject *pyXplusD;
   void *beta_dst;
@@ -57,14 +58,14 @@ void fcn_callback(int *n, int *m, int *np, int *nq, int *ldn, int *ldm,
       dim2[0] = *m;
       dim2[1] = *n;
       pyXplusD = (PyArrayObject *) PyArray_SimpleNew(2, dim2, NPY_DOUBLE);
-      memcpy(pyXplusD->data, (void *)xplusd, (*m) * (*n) * sizeof(double));
+      memcpy(PyArray_DATA(pyXplusD), (void *)xplusd, (*m) * (*n) * sizeof(double));
     }
   else
     {
       npy_intp dim1[1];
       dim1[0] = *n;
       pyXplusD = (PyArrayObject *) PyArray_SimpleNew(1, dim1, NPY_DOUBLE);
-      memcpy(pyXplusD->data, (void *)xplusd, (*n) * sizeof(double));
+      memcpy(PyArray_DATA(pyXplusD), (void *)xplusd, (*n) * sizeof(double));
     }
 
   PyTuple_SetItem(arg01, 0, odr_global.pyBeta);
@@ -84,7 +85,7 @@ void fcn_callback(int *n, int *m, int *np, int *nq, int *ldn, int *ldm,
   Py_DECREF(arg01);
   *istop = 0;
 
-  beta_dst = ((PyArrayObject *) (odr_global.pyBeta))->data;
+  beta_dst = (PyArray_DATA((PyArrayObject *) odr_global.pyBeta));
   if (beta != beta_dst) {
       memcpy(beta_dst, (void *)beta, (*np) * sizeof(double));
   }
@@ -99,7 +100,7 @@ void fcn_callback(int *n, int *m, int *np, int *nq, int *ldn, int *ldm,
           PYERR2(odr_error, "Function has not been initialized");
         }
 
-      if ((result = PyEval_CallObject(odr_global.fcn, arglist)) == NULL)
+      if ((result = PyObject_CallObject(odr_global.fcn, arglist)) == NULL)
         {
           if (PyErr_ExceptionMatches(odr_stop))
             {
@@ -121,7 +122,7 @@ void fcn_callback(int *n, int *m, int *np, int *nq, int *ldn, int *ldm,
                  "Result from function call is not a proper array of floats.");
         }
 
-      memcpy((void *)f, result_array->data, (*n) * (*nq) * sizeof(double));
+      memcpy((void *)f, PyArray_DATA(result_array), (*n) * (*nq) * sizeof(double));
       Py_DECREF(result_array);
     }
 
@@ -135,7 +136,7 @@ void fcn_callback(int *n, int *m, int *np, int *nq, int *ldn, int *ldm,
           PYERR2(odr_error, "Function has not been initialized");
         }
 
-      if ((result = PyEval_CallObject(odr_global.fjacb, arglist)) == NULL)
+      if ((result = PyObject_CallObject(odr_global.fjacb, arglist)) == NULL)
         {
           if (PyErr_ExceptionMatches(odr_stop))
             {
@@ -151,7 +152,7 @@ void fcn_callback(int *n, int *m, int *np, int *nq, int *ldn, int *ldm,
       if ((result_array =
            (PyArrayObject *) PyArray_ContiguousFromObject(result,
                                                           NPY_DOUBLE, 0,
-                                                          2)) == NULL)
+                                                          3)) == NULL)
         {
           PYERR2(odr_error,
                  "Result from function call is not a proper array of floats.");
@@ -161,7 +162,7 @@ void fcn_callback(int *n, int *m, int *np, int *nq, int *ldn, int *ldm,
         {
           /* result_array should be rank-3 */
 
-          if (result_array->nd != 3)
+          if (PyArray_NDIM(result_array) != 3)
             {
               Py_DECREF(result_array);
               PYERR2(odr_error, "Beta Jacobian is not rank-3");
@@ -171,14 +172,14 @@ void fcn_callback(int *n, int *m, int *np, int *nq, int *ldn, int *ldm,
         {
           /* result_array should be rank-2 */
 
-          if (result_array->nd != 2)
+          if (PyArray_NDIM(result_array) != 2)
             {
               Py_DECREF(result_array);
               PYERR2(odr_error, "Beta Jacobian is not rank-2");
             }
         }
 
-      memcpy((void *)fjacb, result_array->data,
+      memcpy((void *)fjacb, PyArray_DATA(result_array),
              (*n) * (*nq) * (*np) * sizeof(double));
       Py_DECREF(result_array);
 
@@ -194,7 +195,7 @@ void fcn_callback(int *n, int *m, int *np, int *nq, int *ldn, int *ldm,
           PYERR2(odr_error, "fjcad has not been initialized");
         }
 
-      if ((result = PyEval_CallObject(odr_global.fjacd, arglist)) == NULL)
+      if ((result = PyObject_CallObject(odr_global.fjacd, arglist)) == NULL)
         {
           if (PyErr_ExceptionMatches(odr_stop))
             {
@@ -210,7 +211,7 @@ void fcn_callback(int *n, int *m, int *np, int *nq, int *ldn, int *ldm,
       if ((result_array =
            (PyArrayObject *) PyArray_ContiguousFromObject(result,
                                                           NPY_DOUBLE, 0,
-                                                          2)) == NULL)
+                                                          3)) == NULL)
         {
           PYERR2(odr_error,
                  "Result from function call is not a proper array of floats.");
@@ -220,7 +221,7 @@ void fcn_callback(int *n, int *m, int *np, int *nq, int *ldn, int *ldm,
         {
           /* result_array should be rank-3 */
 
-          if (result_array->nd != 3)
+          if (PyArray_NDIM(result_array) != 3)
             {
               Py_DECREF(result_array);
               PYERR2(odr_error, "xplusd Jacobian is not rank-3");
@@ -230,7 +231,7 @@ void fcn_callback(int *n, int *m, int *np, int *nq, int *ldn, int *ldm,
         {
           /* result_array should be rank-2 */
 
-          if (result_array->nd != 2)
+          if (PyArray_NDIM(result_array) != 2)
             {
               Py_DECREF(result_array);
               PYERR2(odr_error, "xplusd Jacobian is not rank-2");
@@ -240,14 +241,14 @@ void fcn_callback(int *n, int *m, int *np, int *nq, int *ldn, int *ldm,
         {
           /* result_array should be rank-1 */
 
-          if (result_array->nd != 1)
+          if (PyArray_NDIM(result_array) != 1)
             {
               Py_DECREF(result_array);
               PYERR2(odr_error, "xplusd Jacobian is not rank-1");
             }
         }
 
-      memcpy((void *)fjacd, result_array->data,
+      memcpy((void *)fjacd, PyArray_DATA(result_array),
              (*n) * (*nq) * (*m) * sizeof(double));
       Py_DECREF(result_array);
     }
@@ -268,18 +269,18 @@ fail:
 
 
 /* generates Python output from the raw output from DODRC */
-PyObject *gen_output(int n, int m, int np, int nq, int ldwe, int ld2we,
+PyObject *gen_output(F_INT n, F_INT m, F_INT np, F_INT nq, F_INT ldwe, F_INT ld2we,
                      PyArrayObject * beta, PyArrayObject * work,
-                     PyArrayObject * iwork, int isodr, int info,
+                     PyArrayObject * iwork, F_INT isodr, F_INT info,
                      int full_output)
 {
   PyArrayObject *sd_beta, *cov_beta;
 
-  int delta, eps, xplus, fn, sd, vcv, rvar, wss, wssde, wssep, rcond;
-  int eta, olmav, tau, alpha, actrs, pnorm, rnors, prers, partl, sstol;
-  int taufc, apsma, betao, betac, betas, betan, s, ss, ssf, qraux, u;
-  int fs, fjacb, we1, diff, delts, deltn, t, tt, omega, fjacd;
-  int wrk1, wrk2, wrk3, wrk4, wrk5, wrk6, wrk7, lwkmn;
+  F_INT delta, eps, xplus, fn, sd, vcv, rvar, wss, wssde, wssep, rcond;
+  F_INT eta, olmav, tau, alpha, actrs, pnorm, rnors, prers, partl, sstol;
+  F_INT taufc, apsma, betao, betac, betas, betan, s, ss, ssf, qraux, u;
+  F_INT fs, fjacb, we1, diff, delts, deltn, t, tt, omega, fjacd;
+  F_INT wrk1, wrk2, wrk3, wrk4, wrk5, wrk6, wrk7, lwkmn;
 
   PyObject *retobj;
 
@@ -291,7 +292,7 @@ PyObject *gen_output(int n, int m, int np, int nq, int ldwe, int ld2we,
       return NULL;
   }
 
-  lwkmn = work->dimensions[0];
+  lwkmn = PyArray_DIMS(work)[0];
 
   F_FUNC(dwinf,DWINF)(&n, &m, &np, &nq, &ldwe, &ld2we, &isodr,
         &delta, &eps, &xplus, &fn, &sd, &vcv, &rvar, &wss, &wssde,
@@ -352,15 +353,15 @@ PyObject *gen_output(int n, int m, int np, int nq, int ldwe, int ld2we,
   wrk6--;
   wrk7--;
 
-  dim1[0] = beta->dimensions[0];
+  dim1[0] = PyArray_DIMS(beta)[0];
   sd_beta = (PyArrayObject *) PyArray_SimpleNew(1, dim1, NPY_DOUBLE);
-  dim2[0] = beta->dimensions[0];
-  dim2[1] = beta->dimensions[0];
+  dim2[0] = PyArray_DIMS(beta)[0];
+  dim2[1] = PyArray_DIMS(beta)[0];
   cov_beta = (PyArrayObject *) PyArray_SimpleNew(2, dim2, NPY_DOUBLE);
 
-  memcpy(sd_beta->data, (void *)((double *)(work->data) + sd),
+  memcpy(PyArray_DATA(sd_beta), (void *)((double *)(PyArray_DATA(work)) + sd),
          np * sizeof(double));
-  memcpy(cov_beta->data, (void *)((double *)(work->data) + vcv),
+  memcpy(PyArray_DATA(cov_beta), (void *)((double *)(PyArray_DATA(work)) + vcv),
          np * np * sizeof(double));
 
   if (!full_output)
@@ -382,8 +383,20 @@ PyObject *gen_output(int n, int m, int np, int nq, int ldwe, int ld2we,
 
       work_ind =
         Py_BuildValue
-        ("{s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i}",
-         "delta", delta, "eps", eps, "xplus", xplus, "fn", fn, "sd", sd, "sd",
+          (("{s:" F_INT_PYFMT ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT
+            ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT
+            ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT
+            ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT
+            ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT
+            ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT
+            ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT
+            ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT
+            ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT
+            ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT
+            ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT
+            ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT ",s:" F_INT_PYFMT
+            ",s:" F_INT_PYFMT "}"),
+         "delta", delta, "eps", eps, "xplus", xplus, "fn", fn, "sd", sd, "vcv",
          vcv, "rvar", rvar, "wss", wss, "wssde", wssde, "wssep", wssep,
          "rcond", rcond, "eta", eta, "olmav", olmav, "tau", tau, "alpha",
          alpha, "actrs", actrs, "pnorm", pnorm, "rnors", rnors, "prers",
@@ -427,25 +440,25 @@ PyObject *gen_output(int n, int m, int np, int nq, int ldwe, int ld2we,
           fnA = (PyArrayObject *) PyArray_SimpleNew(2, dim2, NPY_DOUBLE);
         }
 
-      memcpy(deltaA->data, (void *)((double *)(work->data) + delta),
+      memcpy(PyArray_DATA(deltaA), (void *)((double *)(PyArray_DATA(work)) + delta),
              m * n * sizeof(double));
-      memcpy(epsA->data, (void *)((double *)(work->data) + eps),
+      memcpy(PyArray_DATA(epsA), (void *)((double *)(PyArray_DATA(work)) + eps),
              nq * n * sizeof(double));
-      memcpy(xplusA->data, (void *)((double *)(work->data) + xplus),
+      memcpy(PyArray_DATA(xplusA), (void *)((double *)(PyArray_DATA(work)) + xplus),
              m * n * sizeof(double));
-      memcpy(fnA->data, (void *)((double *)(work->data) + fn),
+      memcpy(PyArray_DATA(fnA), (void *)((double *)(PyArray_DATA(work)) + fn),
              nq * n * sizeof(double));
 
-      res_var = *((double *)(work->data) + rvar);
-      sum_square = *((double *)(work->data) + wss);
-      sum_square_delta = *((double *)(work->data) + wssde);
-      sum_square_eps = *((double *)(work->data) + wssep);
-      inv_condnum = *((double *)(work->data) + rcond);
-      rel_error = *((double *)(work->data) + eta);
+      res_var = *((double *)(PyArray_DATA(work)) + rvar);
+      sum_square = *((double *)(PyArray_DATA(work)) + wss);
+      sum_square_delta = *((double *)(PyArray_DATA(work)) + wssde);
+      sum_square_eps = *((double *)(PyArray_DATA(work)) + wssep);
+      inv_condnum = *((double *)(PyArray_DATA(work)) + rcond);
+      rel_error = *((double *)(PyArray_DATA(work)) + eta);
 
       retobj =
         Py_BuildValue
-        ("OOO{s:O,s:O,s:O,s:O,s:d,s:d,s:d,s:d,s:d,s:d,s:O,s:O,s:O,s:i}",
+         (("OOO{s:O,s:O,s:O,s:O,s:d,s:d,s:d,s:d,s:d,s:d,s:O,s:O,s:O,s:" F_INT_PYFMT "}"),
          PyArray_Return(beta), PyArray_Return(sd_beta),
          PyArray_Return(cov_beta), "delta", PyArray_Return(deltaA), "eps",
          PyArray_Return(epsA), "xplus", PyArray_Return(xplusA), "y",
@@ -472,7 +485,7 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
   PyObject *fjacd = NULL, *pifixb = NULL, *pifixx = NULL;
   PyObject *pstpb = NULL, *pstpd = NULL, *psclb = NULL, *pscld = NULL;
   PyObject *pwork = NULL, *piwork = NULL, *extra_args = NULL;
-  int job = 0, ndigit = 0, maxit = -1, iprint = 0;
+  F_INT job = 0, ndigit = 0, maxit = -1, iprint = 0;
   int full_output = 0;
   double taufac = 0.0, sstol = -1.0, partol = -1.0;
   char *errfile = NULL, *rptfile = NULL;
@@ -481,23 +494,25 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
   PyArrayObject *ifixb = NULL, *ifixx = NULL;
   PyArrayObject *stpb = NULL, *stpd = NULL, *sclb = NULL, *scld = NULL;
   PyArrayObject *work = NULL, *iwork = NULL;
-  int n, m, np, nq, ldy, ldx, ldwe, ld2we, ldwd, ld2wd, ldifx;
-  int lunerr = -1, lunrpt = -1, ldstpd, ldscld, lwork, liwork, info = 0;
+  F_INT n, m, np, nq, ldy, ldx, ldwe, ld2we, ldwd, ld2wd, ldifx;
+  F_INT lunerr = -1, lunrpt = -1, ldstpd, ldscld, lwork, liwork, info = 0;
   static char *kw_list[] = { "fcn", "initbeta", "y", "x", "we", "wd", "fjacb",
     "fjacd", "extra_args", "ifixb", "ifixx", "job", "iprint", "errfile",
     "rptfile", "ndigit", "taufac", "sstol", "partol",
     "maxit", "stpb", "stpd", "sclb", "scld", "work",
     "iwork", "full_output", NULL
   };
-  int isodr = 1;
+  F_INT isodr = 1;
   PyObject *result;
   npy_intp dim1[1], dim2[2], dim3[3];
-  int implicit;                 /* flag for implicit model */
+  F_INT implicit;                 /* flag for implicit model */
 
 
   if (kwds == NULL)
     {
-      if (!PyArg_ParseTuple(args, "OOOO|OOOOOOOiiz#z#idddiOOOOOOi:odr",
+        if (!PyArg_ParseTuple(args, ("OOOO|OOOOOOO" F_INT_PYFMT F_INT_PYFMT
+                                     "z#z#" F_INT_PYFMT "ddd" F_INT_PYFMT
+                                     "OOOOOOi:odr"),
                             &fcn, &initbeta, &py, &px, &pwe, &pwd,
                             &fjacb, &fjacd, &extra_args, &pifixb, &pifixx,
                             &job, &iprint, &errfile, &lerrfile, &rptfile,
@@ -511,7 +526,9 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
   else
     {
       if (!PyArg_ParseTupleAndKeywords(args, kwds,
-                                       "OOOO|OOOOOOOiiz#z#idddiOOOOOOi:odr",
+                                       ("OOOO|OOOOOOO" F_INT_PYFMT "" F_INT_PYFMT
+                                        "z#z#" F_INT_PYFMT "ddd" F_INT_PYFMT
+                                        "OOOOOOi:odr"),
                                        kw_list, &fcn, &initbeta, &py, &px,
                                        &pwe, &pwd, &fjacb, &fjacd,
                                        &extra_args, &pifixb, &pifixx, &job,
@@ -535,21 +552,10 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
     {
       PYERR(PyExc_TypeError, "initbeta must be a sequence");
     }
-  if (!PySequence_Check(py))
+  if (!PySequence_Check(py) && !PyNumber_Check(py))
     {
-      /* Checking whether py is an int 
-       *
-       * XXX: PyInt_Check for np.int32 instances does not work on python 2.6 -
-       * we should fix this in numpy, workaround by trying to cast to an int
-       * for now */
-      long val;
-
-      PyErr_Clear();
-      val = PyInt_AsLong(py);
-      if (val == -1 && PyErr_Occurred()) {
-        PYERR(PyExc_TypeError,
-              "y must be a sequence or integer (if model is implicit)");
-      }
+      PYERR(PyExc_TypeError,
+            "y must be a sequence or integer (if model is implicit)");
     }
   if (!PySequence_Check(px))
     {
@@ -623,7 +629,7 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
           PYERR(PyExc_ValueError,
                 "y could not be made into a suitable array");
         }
-      n = y->dimensions[y->nd - 1];     /* pick the last dimension */
+      n = PyArray_DIMS(y)[PyArray_NDIM(y) - 1];     /* pick the last dimension */
       if ((x =
            (PyArrayObject *) PyArray_CopyFromObject(px, NPY_DOUBLE, 1,
                                                     2)) == NULL)
@@ -631,18 +637,18 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
           PYERR(PyExc_ValueError,
                 "x could not be made into a suitable array");
         }
-      if (n != x->dimensions[x->nd - 1])
+      if (n != PyArray_DIMS(x)[PyArray_NDIM(x) - 1])
         {
           PYERR(PyExc_ValueError,
                 "x and y don't have matching numbers of observations");
         }
-      if (y->nd == 1)
+      if (PyArray_NDIM(y) == 1)
         {
           nq = 1;
         }
       else
         {
-          nq = y->dimensions[0];
+          nq = PyArray_DIMS(y)[0];
         }
 
       ldx = ldy = n;
@@ -650,7 +656,7 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
   else
     {                           /* we *do* have an implicit model */
       ldy = 1;
-      nq = (int)PyInt_AsLong(py);
+      nq = (F_INT)PyLong_AsLong(py);
       dim1[0] = 1;
 
       /* initialize y to a dummy array; never referenced */
@@ -664,17 +670,17 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
                 "x could not be made into a suitable array");
         }
 
-      n = x->dimensions[x->nd - 1];
+      n = PyArray_DIMS(x)[PyArray_NDIM(x) - 1];
       ldx = n;
     }
 
-  if (x->nd == 1)
+  if (PyArray_NDIM(x) == 1)
     {
       m = 1;
     }
   else
     {
-      m = x->dimensions[0];
+      m = PyArray_DIMS(x)[0];
     }                           /* x, y */
 
   if ((beta =
@@ -684,14 +690,14 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
       PYERR(PyExc_ValueError,
             "initbeta could not be made into a suitable array");
     }
-  np = beta->dimensions[0];
+  np = PyArray_DIMS(beta)[0];
 
   if (pwe == NULL)
     {
       ldwe = ld2we = 1;
       dim1[0] = n;
       we = (PyArrayObject *) PyArray_SimpleNew(1, dim1, NPY_DOUBLE);
-      ((double *)(we->data))[0] = -1.0;
+      ((double *)(PyArray_DATA(we)))[0] = -1.0;
     }
   else if (PyNumber_Check(pwe) && !PyArray_Check(pwe))
     {
@@ -711,11 +717,11 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
       we = (PyArrayObject *) PyArray_SimpleNew(3, dim3, NPY_DOUBLE);
       if (implicit)
         {
-          ((double *)(we->data))[0] = val;
+          ((double *)(PyArray_DATA(we)))[0] = val;
         }
       else
         {
-          ((double *)(we->data))[0] = -val;
+          ((double *)(PyArray_DATA(we)))[0] = -val;
         }
       ldwe = ld2we = 1;
     }
@@ -730,46 +736,46 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
           PYERR(PyExc_ValueError, "could not convert we to a suitable array");
         }
 
-      if (we->nd == 1 && nq == 1)
+      if (PyArray_NDIM(we) == 1 && nq == 1)
         {
 
           ldwe = n;
           ld2we = 1;
         }
-      else if (we->nd == 1 && we->dimensions[0] == nq)
+      else if (PyArray_NDIM(we) == 1 && PyArray_DIMS(we)[0] == nq)
         {
-          /* we is a rank-1 array with diagonal weightings to be broadcast 
+          /* we is a rank-1 array with diagonal weightings to be broadcast
            * to all observations */
           ldwe = 1;
           ld2we = 1;
         }
-      else if (we->nd == 3 && we->dimensions[0] == nq
-               && we->dimensions[1] == nq && we->dimensions[2] == 1)
+      else if (PyArray_NDIM(we) == 3 && PyArray_DIMS(we)[0] == nq
+               && PyArray_DIMS(we)[1] == nq && PyArray_DIMS(we)[2] == 1)
         {
-          /* we is a rank-3 array with the covariant weightings 
+          /* we is a rank-3 array with the covariant weightings
              to be broadcast to all observations */
           ldwe = 1;
           ld2we = nq;
         }
-      else if (we->nd == 2 && we->dimensions[0] == nq
-               && we->dimensions[1] == nq)
+      else if (PyArray_NDIM(we) == 2 && PyArray_DIMS(we)[0] == nq
+               && PyArray_DIMS(we)[1] == nq)
         {
-          /* we is a rank-2 array with the full covariant weightings 
+          /* we is a rank-2 array with the full covariant weightings
              to be broadcast to all observations */
           ldwe = 1;
           ld2we = nq;
         }
 
-      else if (we->nd == 2 && we->dimensions[0] == nq
-               && we->dimensions[1] == n)
+      else if (PyArray_NDIM(we) == 2 && PyArray_DIMS(we)[0] == nq
+               && PyArray_DIMS(we)[1] == n)
         {
-          /* we is a rank-2 array with the diagonal elements of the 
+          /* we is a rank-2 array with the diagonal elements of the
              covariant weightings for each observation */
           ldwe = n;
           ld2we = 1;
         }
-      else if (we->nd == 3 && we->dimensions[0] == nq
-               && we->dimensions[1] == nq && we->dimensions[2] == n)
+      else if (PyArray_NDIM(we) == 3 && PyArray_DIMS(we)[0] == nq
+               && PyArray_DIMS(we)[1] == nq && PyArray_DIMS(we)[2] == n)
         {
           /* we is the full specification of the covariant weights
              for each observation */
@@ -788,7 +794,7 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
 
       dim1[0] = m;
       wd = (PyArrayObject *) PyArray_SimpleNew(1, dim1, NPY_DOUBLE);
-      ((double *)(wd->data))[0] = -1.0;
+      ((double *)(PyArray_DATA(wd)))[0] = -1.0;
     }
   else if (PyNumber_Check(pwd) && !PyArray_Check(pwd))
     {
@@ -806,7 +812,7 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
       dim3[1] = 1;
       dim3[2] = m;
       wd = (PyArrayObject *) PyArray_SimpleNew(3, dim3, NPY_DOUBLE);
-      ((double *)(wd->data))[0] = -val;
+      ((double *)(PyArray_DATA(wd)))[0] = -val;
       ldwd = ld2wd = 1;
     }
   else if (PySequence_Check(pwd))
@@ -820,46 +826,46 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
           PYERR(PyExc_ValueError, "could not convert wd to a suitable array");
         }
 
-      if (wd->nd == 1 && m == 1)
+      if (PyArray_NDIM(wd) == 1 && m == 1)
         {
           ldwd = n;
           ld2wd = 1;
         }
-      else if (wd->nd == 1 && wd->dimensions[0] == m)
+      else if (PyArray_NDIM(wd) == 1 && PyArray_DIMS(wd)[0] == m)
         {
-          /* wd is a rank-1 array with diagonal weightings to be broadcast 
+          /* wd is a rank-1 array with diagonal weightings to be broadcast
            * to all observations */
           ldwd = 1;
           ld2wd = 1;
         }
 
-      else if (wd->nd == 3 && wd->dimensions[0] == m
-               && wd->dimensions[1] == m && wd->dimensions[2] == 1)
+      else if (PyArray_NDIM(wd) == 3 && PyArray_DIMS(wd)[0] == m
+               && PyArray_DIMS(wd)[1] == m && PyArray_DIMS(wd)[2] == 1)
         {
-          /* wd is a rank-3 array with the covariant wdightings 
+          /* wd is a rank-3 array with the covariant wdightings
              to be broadcast to all observations */
           ldwd = 1;
           ld2wd = m;
         }
-      else if (wd->nd == 2 && wd->dimensions[0] == m
-               && wd->dimensions[1] == m)
+      else if (PyArray_NDIM(wd) == 2 && PyArray_DIMS(wd)[0] == m
+               && PyArray_DIMS(wd)[1] == m)
         {
-          /* wd is a rank-2 array with the full covariant weightings 
+          /* wd is a rank-2 array with the full covariant weightings
              to be broadcast to all observations */
           ldwd = 1;
           ld2wd = m;
         }
 
-      else if (wd->nd == 2 && wd->dimensions[0] == m
-               && wd->dimensions[1] == n)
+      else if (PyArray_NDIM(wd) == 2 && PyArray_DIMS(wd)[0] == m
+               && PyArray_DIMS(wd)[1] == n)
         {
-          /* wd is a rank-2 array with the diagonal elements of the 
+          /* wd is a rank-2 array with the diagonal elements of the
              covariant weightings for each observation */
           ldwd = n;
           ld2wd = 1;
         }
-      else if (wd->nd == 3 && wd->dimensions[0] == m
-               && wd->dimensions[1] == m && wd->dimensions[2] == n)
+      else if (PyArray_NDIM(wd) == 3 && PyArray_DIMS(wd)[0] == m
+               && PyArray_DIMS(wd)[1] == m && PyArray_DIMS(wd)[2] == n)
         {
           /* wd is the full specification of the covariant weights
              for each observation */
@@ -877,22 +883,22 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
   if (pifixb == NULL)
     {
       dim1[0] = np;
-      ifixb = (PyArrayObject *) PyArray_SimpleNew(1, dim1, NPY_INT);
-      *(int *)(ifixb->data) = -1;      /* set first element negative */
+      ifixb = (PyArrayObject *) PyArray_SimpleNew(1, dim1, F_INT_NPY);
+      *(F_INT *)(PyArray_DATA(ifixb)) = -1;      /* set first element negative */
     }
   else
     {
       /* pifixb is a sequence as checked before */
 
       if ((ifixb =
-           (PyArrayObject *) PyArray_CopyFromObject(pifixb, NPY_INT, 1,
+           (PyArrayObject *) PyArray_CopyFromObject(pifixb, F_INT_NPY, 1,
                                                     1)) == NULL)
         {
           PYERR(PyExc_ValueError,
                 "could not convert ifixb to a suitable array");
         }
 
-      if (ifixb->dimensions[0] != np)
+      if (PyArray_DIMS(ifixb)[0] != np)
         {
           PYERR(PyExc_ValueError,
                 "could not convert ifixb to a suitable array");
@@ -903,8 +909,8 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
     {
       dim2[0] = m;
       dim2[1] = 1;
-      ifixx = (PyArrayObject *) PyArray_SimpleNew(2, dim2, NPY_INT);
-      *(int *)(ifixx->data) = -1;      /* set first element negative */
+      ifixx = (PyArrayObject *) PyArray_SimpleNew(2, dim2, F_INT_NPY);
+      *(F_INT *)(PyArray_DATA(ifixx)) = -1;      /* set first element negative */
       ldifx = 1;
     }
   else
@@ -912,23 +918,23 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
       /* pifixx is a sequence as checked before */
 
       if ((ifixx =
-           (PyArrayObject *) PyArray_CopyFromObject(pifixx, NPY_INT, 1,
+           (PyArrayObject *) PyArray_CopyFromObject(pifixx, F_INT_NPY, 1,
                                                     2)) == NULL)
         {
           PYERR(PyExc_ValueError,
                 "could not convert ifixx to a suitable array");
         }
 
-      if (ifixx->nd == 1 && ifixx->dimensions[0] == m)
+      if (PyArray_NDIM(ifixx) == 1 && PyArray_DIMS(ifixx)[0] == m)
         {
           ldifx = 1;
         }
-      else if (ifixx->nd == 1 && ifixx->dimensions[0] == n && m == 1)
+      else if (PyArray_NDIM(ifixx) == 1 && PyArray_DIMS(ifixx)[0] == n && m == 1)
         {
           ldifx = n;
         }
-      else if (ifixx->nd == 2 && ifixx->dimensions[0] == m
-               && ifixx->dimensions[1] == n)
+      else if (PyArray_NDIM(ifixx) == 2 && PyArray_DIMS(ifixx)[0] == m
+               && PyArray_DIMS(ifixx)[1] == n)
         {
           ldifx = n;
         }
@@ -957,14 +963,14 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
     {
       dim1[0] = np;
       stpb = (PyArrayObject *) PyArray_SimpleNew(1, dim1, NPY_DOUBLE);
-      *(double *)(stpb->data) = 0.0;
+      *(double *)(PyArray_DATA(stpb)) = 0.0;
     }
   else                          /* pstpb is a sequence */
     {
       if ((stpb =
            (PyArrayObject *) PyArray_CopyFromObject(pstpb, NPY_DOUBLE, 1,
                                                     1)) == NULL
-          || stpb->dimensions[0] != np)
+          || PyArray_DIMS(stpb)[0] != np)
         {
           PYERR(PyExc_ValueError,
                 "could not convert stpb to a suitable array");
@@ -976,7 +982,7 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
       dim2[0] = 1;
       dim2[1] = m;
       stpd = (PyArrayObject *) PyArray_SimpleNew(2, dim2, NPY_DOUBLE);
-      *(double *)(stpd->data) = 0.0;
+      *(double *)(PyArray_DATA(stpd)) = 0.0;
       ldstpd = 1;
     }
   else
@@ -989,16 +995,16 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
                 "could not convert stpb to a suitable array");
         }
 
-      if (stpd->nd == 1 && stpd->dimensions[0] == m)
+      if (PyArray_NDIM(stpd) == 1 && PyArray_DIMS(stpd)[0] == m)
         {
           ldstpd = 1;
         }
-      else if (stpd->nd == 1 && stpd->dimensions[0] == n && m == 1)
+      else if (PyArray_NDIM(stpd) == 1 && PyArray_DIMS(stpd)[0] == n && m == 1)
         {
           ldstpd = n;
         }
-      else if (stpd->nd == 2 && stpd->dimensions[0] == n &&
-               stpd->dimensions[1] == m)
+      else if (PyArray_NDIM(stpd) == 2 && PyArray_DIMS(stpd)[0] == n &&
+               PyArray_DIMS(stpd)[1] == m)
         {
           ldstpd = n;
         }
@@ -1008,14 +1014,14 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
     {
       dim1[0] = np;
       sclb = (PyArrayObject *) PyArray_SimpleNew(1, dim1, NPY_DOUBLE);
-      *(double *)(sclb->data) = 0.0;
+      *(double *)(PyArray_DATA(sclb)) = 0.0;
     }
   else                          /* psclb is a sequence */
     {
       if ((sclb =
            (PyArrayObject *) PyArray_CopyFromObject(psclb, NPY_DOUBLE, 1,
                                                     1)) == NULL
-          || sclb->dimensions[0] != np)
+          || PyArray_DIMS(sclb)[0] != np)
         {
           PYERR(PyExc_ValueError,
                 "could not convert sclb to a suitable array");
@@ -1027,7 +1033,7 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
       dim2[0] = 1;
       dim2[1] = n;
       scld = (PyArrayObject *) PyArray_SimpleNew(2, dim2, NPY_DOUBLE);
-      *(double *)(scld->data) = 0.0;
+      *(double *)(PyArray_DATA(scld)) = 0.0;
       ldscld = 1;
     }
   else
@@ -1040,16 +1046,16 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
                 "could not convert stpb to a suitable array");
         }
 
-      if (scld->nd == 1 && scld->dimensions[0] == m)
+      if (PyArray_NDIM(scld) == 1 && PyArray_DIMS(scld)[0] == m)
         {
           ldscld = 1;
         }
-      else if (scld->nd == 1 && scld->dimensions[0] == n && m == 1)
+      else if (PyArray_NDIM(scld) == 1 && PyArray_DIMS(scld)[0] == n && m == 1)
         {
           ldscld = n;
         }
-      else if (scld->nd == 2 && scld->dimensions[0] == n &&
-               scld->dimensions[1] == m)
+      else if (PyArray_NDIM(scld) == 2 && PyArray_DIMS(scld)[0] == n &&
+               PyArray_DIMS(scld)[1] == m)
         {
           ldscld = n;
         }
@@ -1110,9 +1116,9 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
           PYERR(PyExc_ValueError,
                 "could not convert work to a suitable array");
         }
-      if (work->dimensions[0] < lwork)
+      if (PyArray_DIMS(work)[0] < lwork)
         {
-            printf("%d %d\n", work->dimensions[0], lwork);
+          printf("%ld %lld\n", PyArray_DIMS(work)[0], (long long)lwork);
           PYERR(PyExc_ValueError, "work is too small");
         }
     }
@@ -1126,14 +1132,14 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
   if (piwork != NULL)
     {
       if ((iwork =
-           (PyArrayObject *) PyArray_CopyFromObject(piwork, NPY_INT, 1,
+           (PyArrayObject *) PyArray_CopyFromObject(piwork, F_INT_NPY, 1,
                                                     1)) == NULL)
         {
           PYERR(PyExc_ValueError,
                 "could not convert iwork to a suitable array");
         }
 
-      if (iwork->dimensions[0] < liwork)
+      if (PyArray_DIMS(iwork)[0] < liwork)
         {
           PYERR(PyExc_ValueError, "iwork is too small");
         }
@@ -1142,10 +1148,10 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
     {
       /* initialize our own iwork array */
       dim1[0] = liwork;
-      iwork = (PyArrayObject *) PyArray_SimpleNew(1, dim1, NPY_INT);
+      iwork = (PyArrayObject *) PyArray_SimpleNew(1, dim1, F_INT_NPY);
     }                           /* iwork */
 
-  /* check if what JOB requests can be done with what the user has 
+  /* check if what JOB requests can be done with what the user has
      input into the function */
 
   if ((job / 10) % 10 >= 2)
@@ -1170,19 +1176,18 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
   Py_INCREF(beta);
   odr_global.extra_args = extra_args;
   Py_XINCREF(extra_args);
-
-  /* now call DODRC */
-  F_FUNC(dodrc,DODRC)(fcn_callback, &n, &m, &np, &nq, (double *)(beta->data),
-        (double *)(y->data), &ldy, (double *)(x->data), &ldx,
-        (double *)(we->data), &ldwe, &ld2we,
-        (double *)(wd->data), &ldwd, &ld2wd,
-        (int *)(ifixb->data), (int *)(ifixx->data), &ldifx,
-        &job, &ndigit, &taufac, &sstol, &partol, &maxit,
-        &iprint, &lunerr, &lunrpt,
-        (double *)(stpb->data), (double *)(stpd->data), &ldstpd,
-        (double *)(sclb->data), (double *)(scld->data), &ldscld,
-        (double *)(work->data), &lwork, (int *)(iwork->data), &liwork,
-        &info);
+   /* now call DODRC */
+   F_FUNC(dodrc,DODRC)(fcn_callback, &n, &m, &np, &nq, (double *)(PyArray_DATA(beta)),
+         (double *)(PyArray_DATA(y)), &ldy, (double *)(PyArray_DATA(x)), &ldx,
+         (double *)(PyArray_DATA(we)), &ldwe, &ld2we,
+         (double *)(PyArray_DATA(wd)), &ldwd, &ld2wd,
+         (F_INT *)(PyArray_DATA(ifixb)), (F_INT *)(PyArray_DATA(ifixx)), &ldifx,
+         &job, &ndigit, &taufac, &sstol, &partol, &maxit,
+         &iprint, &lunerr, &lunrpt,
+         (double *)(PyArray_DATA(stpb)), (double *)(PyArray_DATA(stpd)), &ldstpd,
+         (double *)(PyArray_DATA(sclb)), (double *)(PyArray_DATA(scld)), &ldscld,
+         (double *)(PyArray_DATA(work)), &lwork, (F_INT *)(PyArray_DATA(iwork)), &liwork,
+         &info);
 
   result = gen_output(n, m, np, nq, ldwe, ld2we,
                       beta, work, iwork, isodr, info, full_output);
@@ -1276,7 +1281,6 @@ static PyMethodDef methods[] = {
   {NULL, NULL},
 };
 
-#if PY_VERSION_HEX >= 0x03000000
 static struct PyModuleDef moduledef = {
     PyModuleDef_HEAD_INIT,
     "_odrpack",
@@ -1296,11 +1300,3 @@ PyObject *PyInit___odrpack(void)
     m = PyModule_Create(&moduledef);
     return m;
 }
-#else
-PyMODINIT_FUNC init__odrpack(void)
-{
-    PyObject *m;
-    import_array();
-    m = Py_InitModule("__odrpack", methods);
-}
-#endif
