@@ -28,7 +28,6 @@ THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#include <math.h>
 #include "numpy/arrayobject.h"
 #include "numpy/ndarraytypes.h"
 #include "rectangular_lsap/rectangular_lsap.h"
@@ -40,35 +39,32 @@ calculate_assignment(PyObject* self, PyObject* args)
     PyObject* b = NULL;
     PyObject* result = NULL;
     PyObject* obj_cost = NULL;
-    if (!PyArg_ParseTuple(args, "O", &obj_cost))
+    int maximize = 0;
+    if (!PyArg_ParseTuple(args, "Op", &obj_cost, &maximize))
         return NULL;
 
     PyArrayObject* obj_cont =
-      (PyArrayObject*)PyArray_ContiguousFromAny(obj_cost, NPY_DOUBLE, 2, 2);
+      (PyArrayObject*)PyArray_ContiguousFromAny(obj_cost, NPY_DOUBLE, 0, 0);
     if (!obj_cont) {
-        PyErr_SetString(PyExc_TypeError, "invalid cost matrix object");
         return NULL;
     }
 
+    if (PyArray_NDIM(obj_cont) != 2) {
+        PyErr_Format(PyExc_ValueError,
+                     "expected a matrix (2-D array), got a %d array",
+                     PyArray_NDIM(obj_cont));
+        goto cleanup;
+    }
+
     double* cost_matrix = (double*)PyArray_DATA(obj_cont);
     if (cost_matrix == NULL) {
         PyErr_SetString(PyExc_TypeError, "invalid cost matrix object");
         goto cleanup;
     }
 
-    int num_rows = PyArray_DIM(obj_cont, 0);
-    int num_cols = PyArray_DIM(obj_cont, 1);
-
-    // test for NaN and -inf entries
-    for (size_t i=0;i<(size_t)num_rows*num_cols;i++) {
-        if (cost_matrix[i] != cost_matrix[i] || cost_matrix[i] == -INFINITY) {
-            PyErr_SetString(PyExc_ValueError,
-                            "matrix contains invalid numeric entries");
-            goto cleanup;
-        }
-    }
-
-    npy_intp dim[1] = { num_rows };
+    npy_intp num_rows = PyArray_DIM(obj_cont, 0);
+    npy_intp num_cols = PyArray_DIM(obj_cont, 1);
+    npy_intp dim[1] = { num_rows < num_cols ? num_rows : num_cols };
     a = PyArray_SimpleNew(1, dim, NPY_INT64);
     if (!a)
         goto cleanup;
@@ -77,16 +73,19 @@ calculate_assignment(PyObject* self, PyObject* args)
     if (!b)
         goto cleanup;
 
-    int64_t* adata = PyArray_DATA((PyArrayObject*)a);
-    for (int i=0;i<num_rows;i++)
-        adata[i] = i;
-
     int ret = solve_rectangular_linear_sum_assignment(
-      num_rows, num_cols, cost_matrix, PyArray_DATA((PyArrayObject*)b));
-    if (ret != 0) {
+      num_rows, num_cols, cost_matrix, maximize,
+      PyArray_DATA((PyArrayObject*)a),
+      PyArray_DATA((PyArrayObject*)b));
+    if (ret == RECTANGULAR_LSAP_INFEASIBLE) {
         PyErr_SetString(PyExc_ValueError, "cost matrix is infeasible");
         goto cleanup;
     }
+    else if (ret == RECTANGULAR_LSAP_INVALID) {
+        PyErr_SetString(PyExc_ValueError,
+                        "matrix contains invalid numeric entries");
+        goto cleanup;
+    }
 
     result = Py_BuildValue("OO", a, b);
 
