@@ -1,4 +1,3 @@
-from __future__ import division, print_function, absolute_import
 import time
 import numpy as np
 from scipy.sparse.linalg import LinearOperator
@@ -6,7 +5,7 @@ from .._differentiable_functions import VectorFunction
 from .._constraints import (
     NonlinearConstraint, LinearConstraint, PreparedConstraint, strict_bounds)
 from .._hessian_update_strategy import BFGS
-from ..optimize import OptimizeResult
+from .._optimize import OptimizeResult
 from .._differentiable_functions import ScalarFunction
 from .equality_constrained_sqp import equality_constrained_sqp
 from .canonical_constraint import (CanonicalConstraint,
@@ -19,11 +18,11 @@ TERMINATION_MESSAGES = {
     0: "The maximum number of function evaluations is exceeded.",
     1: "`gtol` termination condition is satisfied.",
     2: "`xtol` termination condition is satisfied.",
-    3: "`callback` function requested termination"
+    3: "`callback` function requested termination."
 }
 
 
-class HessianLinearOperator(object):
+class HessianLinearOperator:
     """Build LinearOperator from hessp"""
     def __init__(self, hessp, n):
         self.hessp = hessp
@@ -36,7 +35,7 @@ class HessianLinearOperator(object):
         return LinearOperator((self.n, self.n), matvec=matvec)
 
 
-class LagrangianHessian(object):
+class LagrangianHessian:
     """The Hessian of the Lagrangian as LinearOperator.
 
     The Lagrangian is computed as the objective function plus all the
@@ -130,7 +129,7 @@ def _minimize_trustregion_constr(fun, x0, args, grad,
     ----------
     gtol : float, optional
         Tolerance for termination by the norm of the Lagrangian gradient.
-        The algorithm will terminate when both the infinity norm (i.e. max
+        The algorithm will terminate when both the infinity norm (i.e., max
         abs value) of the Lagrangian gradient and the constraint violation
         are smaller than ``gtol``. Default is 1e-8.
     xtol : float, optional
@@ -140,7 +139,7 @@ def _minimize_trustregion_constr(fun, x0, args, grad,
         Default is 1e-8.
     barrier_tol : float, optional
         Threshold on the barrier parameter for the algorithm termination.
-        When inequality constraints are present the algorithm will terminate
+        When inequality constraints are present, the algorithm will terminate
         only when the barrier parameter is less than `barrier_tol`.
         Default is 1e-8.
     sparse_jacobian : {bool, None}, optional
@@ -177,10 +176,12 @@ def _minimize_trustregion_constr(fun, x0, args, grad,
         ``c(x) <= 0`` the algorithm introduces slack variables, solving the problem
         ``min_(x,s) f(x) + barrier_parameter*sum(ln(s))`` subject to the equality
         constraints  ``c(x) + s = 0`` instead of the original problem. This subproblem
-        is solved for increasing values of ``barrier_parameter`` and with decreasing
+        is solved for decreasing values of ``barrier_parameter`` and with decreasing
         tolerances for the termination, starting with ``initial_barrier_parameter``
         for the barrier parameter and ``initial_barrier_tolerance`` for the
-        barrier subproblem  barrier. Default is 0.1 for both values (recommended in [1]_ p. 19).
+        barrier tolerance. Default is 0.1 for both values (recommended in [1]_ p. 19).
+        Also note that ``barrier_parameter`` and ``barrier_tolerance`` are updated
+        with the same prefactor.
     factorization_method : string or None, optional
         Method to factorize the Jacobian of the constraints. Use None (default)
         for the auto selection or one of:
@@ -205,7 +206,7 @@ def _minimize_trustregion_constr(fun, x0, args, grad,
         method can cope with Jacobian matrices with deficient row rank and will
         be used whenever other factorization methods fail (which may imply the
         conversion of sparse matrices to a dense format when required).
-        By default 'QRFactorization' is used for dense matrices.
+        By default, 'QRFactorization' is used for dense matrices.
     finite_diff_rel_step : None or array_like, optional
         Relative step size for the finite difference approximation.
     maxiter : int, optional
@@ -219,7 +220,7 @@ def _minimize_trustregion_constr(fun, x0, args, grad,
             * 3 : display progress during iterations (more complete report).
 
     disp : bool, optional
-        If True (default) then `verbose` will be set to 1 if it was 0.
+        If True (default), then `verbose` will be set to 1 if it was 0.
 
     Returns
     -------
@@ -230,7 +231,7 @@ def _minimize_trustregion_constr(fun, x0, args, grad,
            constraints are put *after* other constraints.
         2. All numbers of function, Jacobian or Hessian evaluations correspond
            to numbers of actual Python function calls. It means, for example,
-           that if a Jacobian is estimated by finite differences then the
+           that if a Jacobian is estimated by finite differences, then the
            number of Jacobian evaluations will be zero and the number of
            function evaluations will be incremented by all calls during the
            finite difference estimation.
@@ -251,7 +252,7 @@ def _minimize_trustregion_constr(fun, x0, args, grad,
         Total number of iterations.
     nfev : integer
         Number of the objective function evaluations.
-    ngev : integer
+    njev : integer
         Number of the objective function gradient evaluations.
     nhev : integer
         Number of the objective function Hessian evaluations.
@@ -331,7 +332,7 @@ def _minimize_trustregion_constr(fun, x0, args, grad,
     objective = ScalarFunction(fun, x0, args, grad, hess,
                                finite_diff_rel_step, finite_diff_bounds)
 
-    # Put constraints in list format when needed
+    # Put constraints in list format when needed.
     if isinstance(constraints, (NonlinearConstraint, LinearConstraint)):
         constraints = [constraints]
 
@@ -433,7 +434,7 @@ def _minimize_trustregion_constr(fun, x0, args, grad,
                 state.status = 1
             elif state.tr_radius < xtol:
                 state.status = 2
-            elif state.nit > maxiter:
+            elif state.nit >= maxiter:
                 state.status = 0
             return state.status in (0, 1, 2, 3)
     elif method == 'tr_interior_point':
@@ -464,7 +465,7 @@ def _minimize_trustregion_constr(fun, x0, args, grad,
                                          state.barrier_parameter,
                                          state.cg_stop_cond)
             state.status = None
-            state.niter = state.nit  # Alias for callback (backward-compatibility)
+            state.niter = state.nit  # Alias for callback (backward compatibility)
             if callback is not None and callback(np.copy(state.x), state):
                 state.status = 3
             elif state.optimality < gtol and state.constr_violation < gtol:
@@ -472,7 +473,7 @@ def _minimize_trustregion_constr(fun, x0, args, grad,
             elif (state.tr_radius < xtol
                   and state.barrier_parameter < barrier_tol):
                 state.status = 2
-            elif state.nit > maxiter:
+            elif state.nit >= maxiter:
                 state.status = 0
             return state.status in (0, 1, 2, 3)
 
