@@ -21,7 +21,7 @@ from pytest import raises as assert_raises, warns
 import pytest
 
 
-class TestDifferentialEvolutionSolver(object):
+class TestDifferentialEvolutionSolver:
 
     def setup_method(self):
         self.old_seterr = np.seterr(invalid='raise')
@@ -195,6 +195,15 @@ class TestDifferentialEvolutionSolver(object):
         assert_equal(0.5, solver.scale)
         assert_equal(None, solver.dither)
 
+    def test_invalid_functional(self):
+        def func(x):
+            return np.array([np.sum(x ** 2), np.sum(x)])
+
+        with assert_raises(
+                RuntimeError,
+                match=r"func\(x, \*args\) must return a scalar value"):
+            differential_evolution(func, [(-2, 2), (-2, 2)])
+
     def test__scale_parameters(self):
         trial = np.array([0.3])
         assert_equal(30, self.dummy_solver._scale_parameters(trial))
@@ -346,23 +355,25 @@ class TestDifferentialEvolutionSolver(object):
         assert_equal(result.nfev, 41)
         assert_equal(result.success, False)
         assert_equal(result.message,
-                         'Maximum number of function evaluations has '
-                              'been exceeded.')
+                     'Maximum number of function evaluations has '
+                     'been exceeded.')
 
         # now repeat for updating='deferred version
+        # 47 function evaluations is not a multiple of the population size,
+        # so maxfun is reached partway through a population evaluation.
         solver = DifferentialEvolutionSolver(rosen,
                                              self.bounds,
                                              popsize=5,
                                              polish=False,
-                                             maxfun=40,
+                                             maxfun=47,
                                              updating='deferred')
         result = solver.solve()
 
-        assert_equal(result.nfev, 40)
+        assert_equal(result.nfev, 47)
         assert_equal(result.success, False)
         assert_equal(result.message,
-                         'Maximum number of function evaluations has '
-                              'been reached.')
+                     'Maximum number of function evaluations has '
+                     'been reached.')
 
     def test_quadratic(self):
         # test the quadratic function from object
@@ -399,7 +410,7 @@ class TestDifferentialEvolutionSolver(object):
         # obtain a np.random.Generator object
         rng = np.random.default_rng()
 
-        inits = ['random', 'latinhypercube']
+        inits = ['random', 'latinhypercube', 'sobol', 'halton']
         for init in inits:
             differential_evolution(self.quadratic,
                                    [(-100, 100)],
@@ -507,6 +518,15 @@ class TestDifferentialEvolutionSolver(object):
         assert_equal(solver._nfev, 0)
         assert_(np.all(np.isinf(solver.population_energies)))
 
+        solver.init_population_qmc(qmc_engine='halton')
+        assert_equal(solver._nfev, 0)
+        assert_(np.all(np.isinf(solver.population_energies)))
+
+        solver = DifferentialEvolutionSolver(rosen, self.bounds, init='sobol')
+        solver.init_population_qmc(qmc_engine='sobol')
+        assert_equal(solver._nfev, 0)
+        assert_(np.all(np.isinf(solver.population_energies)))
+
         # we should be able to initialize with our own array
         population = np.linspace(-1, 3, 10).reshape(5, 2)
         solver = DifferentialEvolutionSolver(rosen, self.bounds,
@@ -536,6 +556,24 @@ class TestDifferentialEvolutionSolver(object):
                       *(rosen, self.bounds),
                       **{'init': population})
 
+        # provide an initial solution
+        # bounds are [(0, 2), (0, 2)]
+        x0 = np.random.uniform(low=0.0, high=2.0, size=2)
+        solver = DifferentialEvolutionSolver(
+            rosen, self.bounds, x0=x0
+        )
+        # parameters are scaled to unit interval
+        assert_allclose(solver.population[0], x0 / 2.0)
+
+    def test_x0(self):
+        # smoke test that checks that x0 is usable.
+        res = differential_evolution(rosen, self.bounds, x0=[0.2, 0.8])
+        assert res.success
+
+        # check what happens if some of the x0 lay outside the bounds
+        with assert_raises(ValueError):
+            differential_evolution(rosen, self.bounds, x0=[0.2, 2.1])
+
     def test_infinite_objective_function(self):
         # Test that there are no problems if the objective function
         # returns inf on some runs
