@@ -1,4 +1,3 @@
-from __future__ import division, print_function, absolute_import
 import pytest
 import numpy as np
 from numpy.testing import (TestCase, assert_array_almost_equal,
@@ -10,6 +9,7 @@ from scipy.optimize._differentiable_functions import (ScalarFunction,
                                                       VectorFunction,
                                                       LinearVectorFunction,
                                                       IdentityVectorFunction)
+from scipy.optimize import rosen, rosen_der, rosen_hess
 from scipy.optimize._hessian_update_strategy import BFGS
 
 
@@ -52,9 +52,9 @@ class TestScalarFunction(TestCase):
         approx = ScalarFunction(ex.fun, x0, (), '2-point',
                                 ex.hess, None, (-np.inf, np.inf))
         nfev += 3
+        ngev += 1
         assert_array_equal(ex.nfev, nfev)
         assert_array_equal(analit.nfev+approx.nfev, nfev)
-        assert_array_equal(ex.ngev, ngev)
         assert_array_equal(analit.ngev+approx.ngev, ngev)
         assert_array_equal(analit.f, approx.f)
         assert_array_almost_equal(analit.g, approx.g)
@@ -66,14 +66,13 @@ class TestScalarFunction(TestCase):
         ngev += 1
         assert_array_equal(ex.nfev, nfev)
         assert_array_equal(analit.nfev+approx.nfev, nfev)
-        assert_array_equal(ex.ngev, ngev)
         assert_array_equal(analit.ngev+approx.ngev, ngev)
         f_approx = approx.fun(x)
         g_approx = approx.grad(x)
         nfev += 3
+        ngev += 1
         assert_array_equal(ex.nfev, nfev)
         assert_array_equal(analit.nfev+approx.nfev, nfev)
-        assert_array_equal(ex.ngev, ngev)
         assert_array_equal(analit.ngev+approx.ngev, ngev)
         assert_array_almost_equal(f_analit, f_approx)
         assert_array_almost_equal(g_analit, g_approx)
@@ -83,14 +82,13 @@ class TestScalarFunction(TestCase):
         ngev += 1
         assert_array_equal(ex.nfev, nfev)
         assert_array_equal(analit.nfev+approx.nfev, nfev)
-        assert_array_equal(ex.ngev, ngev)
         assert_array_equal(analit.ngev+approx.ngev, ngev)
 
         g_approx = approx.grad(x)
         nfev += 3
+        ngev += 1
         assert_array_equal(ex.nfev, nfev)
         assert_array_equal(analit.nfev+approx.nfev, nfev)
-        assert_array_equal(ex.ngev, ngev)
         assert_array_equal(analit.ngev+approx.ngev, ngev)
         assert_array_almost_equal(g_analit, g_approx)
 
@@ -101,14 +99,13 @@ class TestScalarFunction(TestCase):
         ngev += 1
         assert_array_equal(ex.nfev, nfev)
         assert_array_equal(analit.nfev+approx.nfev, nfev)
-        assert_array_equal(ex.ngev, ngev)
         assert_array_equal(analit.ngev+approx.ngev, ngev)
         f_approx = approx.fun(x)
         g_approx = approx.grad(x)
         nfev += 3
+        ngev += 1
         assert_array_equal(ex.nfev, nfev)
         assert_array_equal(analit.nfev+approx.nfev, nfev)
-        assert_array_equal(ex.ngev, ngev)
         assert_array_equal(analit.ngev+approx.ngev, ngev)
         assert_array_almost_equal(f_analit, f_approx)
         assert_array_almost_equal(g_analit, g_approx)
@@ -120,18 +117,50 @@ class TestScalarFunction(TestCase):
         ngev += 1
         assert_array_equal(ex.nfev, nfev)
         assert_array_equal(analit.nfev+approx.nfev, nfev)
-        assert_array_equal(ex.ngev, ngev)
         assert_array_equal(analit.ngev+approx.ngev, ngev)
         f_approx = approx.fun(x)
         g_approx = approx.grad(x)
         nfev += 3
+        ngev += 1
         assert_array_equal(ex.nfev, nfev)
         assert_array_equal(analit.nfev+approx.nfev, nfev)
-        assert_array_equal(ex.ngev, ngev)
         assert_array_equal(analit.ngev+approx.ngev, ngev)
         assert_array_almost_equal(f_analit, f_approx)
         assert_array_almost_equal(g_analit, g_approx)
 
+    def test_fun_and_grad(self):
+        ex = ExScalarFunction()
+
+        def fg_allclose(x, y):
+            assert_allclose(x[0], y[0])
+            assert_allclose(x[1], y[1])
+
+        # with analytic gradient
+        x0 = [2.0, 0.3]
+        analit = ScalarFunction(ex.fun, x0, (), ex.grad,
+                                ex.hess, None, (-np.inf, np.inf))
+
+        fg = ex.fun(x0), ex.grad(x0)
+        fg_allclose(analit.fun_and_grad(x0), fg)
+        assert(analit.ngev == 1)
+
+        x0[1] = 1.
+        fg = ex.fun(x0), ex.grad(x0)
+        fg_allclose(analit.fun_and_grad(x0), fg)
+
+        # with finite difference gradient
+        x0 = [2.0, 0.3]
+        sf = ScalarFunction(ex.fun, x0, (), '3-point',
+                                ex.hess, None, (-np.inf, np.inf))
+        assert(sf.ngev == 1)
+        fg = ex.fun(x0), ex.grad(x0)
+        fg_allclose(sf.fun_and_grad(x0), fg)
+        assert(sf.ngev == 1)
+
+        x0[1] = 1.
+        fg = ex.fun(x0), ex.grad(x0)
+        fg_allclose(sf.fun_and_grad(x0), fg)
+
     def test_finite_difference_hess_linear_operator(self):
         ex = ExScalarFunction()
         nfev = 0
@@ -298,6 +327,41 @@ class TestScalarFunction(TestCase):
         assert_equal(f2, 14.0)
         assert x is not sf.x
 
+        # gh13740 x is changed in user function
+        def ff(x):
+            x *= x    # overwrite x
+            return np.sum(x)
+
+        x = np.array([1., 2., 3.])
+        sf = ScalarFunction(
+            ff, x, (), '3-point', lambda x: x, None, (-np.inf, np.inf)
+        )
+        assert x is not sf.x
+        assert_equal(sf.fun(x), 14.0)
+        assert_equal(sf.x, np.array([1., 2., 3.]))
+        assert x is not sf.x
+
+    def test_lowest_x(self):
+        # ScalarFunction should remember the lowest func(x) visited.
+        x0 = np.array([2, 3, 4])
+        sf = ScalarFunction(rosen, x0, (), rosen_der, rosen_hess,
+                            None, None)
+        sf.fun([1, 1, 1])
+        sf.fun(x0)
+        sf.fun([1.01, 1, 1.0])
+        sf.grad([1.01, 1, 1.0])
+        assert_equal(sf._lowest_f, 0.0)
+        assert_equal(sf._lowest_x, [1.0, 1.0, 1.0])
+
+        sf = ScalarFunction(rosen, x0, (), '2-point', rosen_hess,
+                            None, (-np.inf, np.inf))
+        sf.fun([1, 1, 1])
+        sf.fun(x0)
+        sf.fun([1.01, 1, 1.0])
+        sf.grad([1.01, 1, 1.0])
+        assert_equal(sf._lowest_f, 0.0)
+        assert_equal(sf._lowest_x, [1.0, 1.0, 1.0])
+
 
 class ExVectorialFunction:
 
@@ -330,7 +394,6 @@ class TestVectorialFunction(TestCase):
         njev = 0
 
         x0 = [1.0, 0.0]
-        v0 = [0.0, 1.0]
         analit = VectorFunction(ex.fun, x0, ex.jac, ex.hess, None, None,
                                 (-np.inf, np.inf), None)
         nfev += 1
