@@ -1,22 +1,17 @@
-from __future__ import division, print_function, absolute_import
-
 import warnings
 
 import numpy as np
 from numpy.testing import (assert_almost_equal, assert_equal, assert_allclose,
-                           assert_)
+                           assert_, suppress_warnings)
 from pytest import raises as assert_raises
 
-from scipy._lib._numpy_compat import suppress_warnings
 from scipy.signal import (ss2tf, tf2ss, lsim2, impulse2, step2, lti,
                           dlti, bode, freqresp, lsim, impulse, step,
                           abcd_normalize, place_poles,
                           TransferFunction, StateSpace, ZerosPolesGain)
-from scipy.signal.filter_design import BadCoefficients
+from scipy.signal._filter_design import BadCoefficients
 import scipy.linalg as linalg
-from scipy.sparse.sputils import matrix
-
-import scipy._lib.six as six
+from scipy.sparse._sputils import matrix
 
 
 def _assert_poles_close(P1,P2, rtol=1e-8, atol=1e-8):
@@ -40,7 +35,7 @@ def _assert_poles_close(P1,P2, rtol=1e-8, atol=1e-8):
             raise ValueError("Can't find pole " + str(p1) + " in " + str(P2))
 
 
-class TestPlacePoles(object):
+class TestPlacePoles:
 
     def _check(self, A, B, P, **kwargs):
         """
@@ -332,7 +327,7 @@ class TestSS2TF:
         assert_allclose(num, [[0, 1, 2, 3], [0, 1, 2, 3]], rtol=1e-13)
         assert_allclose(den, [1, 2, 3, 4], rtol=1e-13)
 
-        tf = ([1, [2, 3]], [1, 6])
+        tf = (np.array([1, [2, 3]], dtype=object), [1, 6])
         A, B, C, D = tf2ss(*tf)
         assert_allclose(A, [[-6]], rtol=1e-31)
         assert_allclose(B, [[1]], rtol=1e-31)
@@ -343,7 +338,7 @@ class TestSS2TF:
         assert_allclose(num, [[0, 1], [2, 3]], rtol=1e-13)
         assert_allclose(den, [1, 6], rtol=1e-13)
 
-        tf = ([[1, -3], [1, 2, 3]], [1, 6, 5])
+        tf = (np.array([[1, -3], [1, 2, 3]], dtype=object), [1, 6, 5])
         A, B, C, D = tf2ss(*tf)
         assert_allclose(A, [[-6, -5], [1, 0]], rtol=1e-13)
         assert_allclose(B, [[1], [0]], rtol=1e-13)
@@ -354,6 +349,15 @@ class TestSS2TF:
         assert_allclose(num, [[0, 1, -3], [1, 2, 3]], rtol=1e-13)
         assert_allclose(den, [1, 6, 5], rtol=1e-13)
 
+    def test_all_int_arrays(self):
+        A = [[0, 1, 0], [0, 0, 1], [-3, -4, -2]]
+        B = [[0], [0], [1]]
+        C = [[5, 1, 0]]
+        D = [[0]]
+        num, den = ss2tf(A, B, C, D)
+        assert_allclose(num, [[0.0, 0.0, 1.0, 5.0]], rtol=1e-13, atol=1e-14)
+        assert_allclose(den, [1.0, 2.0, 4.0, 3.0], rtol=1e-13)
+
     def test_multioutput(self):
         # Regression test for gh-2669.
 
@@ -393,7 +397,7 @@ class TestSS2TF:
         assert_allclose(b_all, np.vstack((b0, b1, b2)), rtol=1e-13, atol=1e-14)
 
 
-class TestLsim(object):
+class TestLsim:
     def lti_nowarn(self, *args):
         with suppress_warnings() as sup:
             sup.filter(BadCoefficients)
@@ -423,9 +427,9 @@ class TestLsim(object):
 
     def test_double_integrator(self):
         # double integrator: y'' = 2u
-        A = matrix("0. 1.; 0. 0.")
-        B = matrix("0.; 1.")
-        C = matrix("2. 0.")
+        A = matrix([[0., 1.], [0., 0.]])
+        B = matrix([[0.], [1.]])
+        C = matrix([[2., 0.]])
         system = self.lti_nowarn(A, B, C, 0.)
         t = np.linspace(0,5)
         u = np.ones_like(t)
@@ -441,9 +445,9 @@ class TestLsim(object):
         #   x2' + x2 = u
         #   y = x1
         # Exact solution with u = 0 is y(t) = t exp(-t)
-        A = matrix("-1. 1.; 0. -1.")
-        B = matrix("0.; 1.")
-        C = matrix("1. 0.")
+        A = matrix([[-1., 1.], [0., -1.]])
+        B = matrix([[0.], [1.]])
+        C = matrix([[1., 0.]])
         system = self.lti_nowarn(A, B, C, 0.)
         t = np.linspace(0,5)
         u = np.zeros_like(t)
@@ -478,7 +482,7 @@ class TestLsim(object):
         assert_almost_equal(y, expected_y)
 
 
-class Test_lsim2(object):
+class Test_lsim2:
 
     def test_01(self):
         t = np.linspace(0,10,1001)
@@ -521,7 +525,7 @@ class Test_lsim2(object):
 
     def test_05(self):
         # The call to lsim2 triggers a "BadCoefficients" warning from
-        # scipy.signal.filter_design, but the test passes.  I think the warning
+        # scipy.signal._filter_design, but the test passes.  I think the warning
         # is related to the incomplete handling of multi-input systems in
         # scipy.signal.
 
@@ -553,7 +557,7 @@ class Test_lsim2(object):
         assert_almost_equal(x[:,0], expected_x)
 
 
-class _TestImpulseFuncs(object):
+class _TestImpulseFuncs:
     # Common tests for impulse/impulse2 (= self.func)
 
     def test_01(self):
@@ -636,7 +640,7 @@ class TestImpulse(_TestImpulseFuncs):
         self.func = impulse
 
 
-class _TestStepFuncs(object):
+class _TestStepFuncs:
     def test_01(self):
         # First order system: x'(t) + x(t) = u(t)
         # Exact step response is x(t) = 1 - exp(-t).
@@ -731,7 +735,7 @@ class TestStep(_TestStepFuncs):
         step(([], [-1], 1+0j))
 
 
-class TestLti(object):
+class TestLti:
     def test_lti_instantiation(self):
         # Test that lti can be instantiated with sequences, scalars.
         # See PR-225.
@@ -759,13 +763,13 @@ class TestLti(object):
         assert_(s.dt is None)
 
 
-class TestStateSpace(object):
+class TestStateSpace:
     def test_initialization(self):
         # Check that all initializations work
-        s = StateSpace(1, 1, 1, 1)
-        s = StateSpace([1], [2], [3], [4])
-        s = StateSpace(np.array([[1, 2], [3, 4]]), np.array([[1], [2]]),
-                       np.array([[1, 0]]), np.array([[0]]))
+        StateSpace(1, 1, 1, 1)
+        StateSpace([1], [2], [3], [4])
+        StateSpace(np.array([[1, 2], [3, 4]]), np.array([[1], [2]]),
+                   np.array([[1, 0]]), np.array([[0]]))
 
     def test_conversion(self):
         # Check the conversion functions
@@ -791,7 +795,7 @@ class TestStateSpace(object):
     def test_operators(self):
         # Test +/-/* operators on systems
 
-        class BadType(object):
+        class BadType:
             pass
 
         s1 = StateSpace(np.array([[-0.5, 0.7], [0.3, -0.8]]),
@@ -808,6 +812,7 @@ class TestStateSpace(object):
 
         s_discrete = s1.to_discrete(0.1)
         s2_discrete = s2.to_discrete(0.2)
+        s3_discrete = s2.to_discrete(0.1)
 
         # Impulse response
         t = np.linspace(0, 1, 100)
@@ -815,8 +820,7 @@ class TestStateSpace(object):
         u[0] = 1
 
         # Test multiplication
-        for typ in six.integer_types + (float, complex, np.float32,
-                                        np.complex128, np.array):
+        for typ in (int, float, complex, np.float32, np.complex128, np.array):
             assert_allclose(lsim(typ(2) * s1, U=u, T=t)[1],
                             typ(2) * lsim(s1, U=u, T=t)[1])
 
@@ -888,7 +892,7 @@ class TestStateSpace(object):
         assert_allclose(lsim(s1 + s2, U=u, T=t)[1],
                         lsim(s1, U=u, T=t)[1] + lsim(s2, U=u, T=t)[1])
 
-        # Test substraction
+        # Test subtraction
         assert_allclose(lsim(s1 - 2, U=u, T=t)[1],
                         -2 * u + lsim(s1, U=u, T=t)[1])
 
@@ -904,13 +908,24 @@ class TestStateSpace(object):
         with assert_raises(TypeError):
             BadType() - s1
 
+        s = s_discrete + s3_discrete
+        assert_(s.dt == 0.1)
 
-class TestTransferFunction(object):
+        s = s_discrete * s3_discrete
+        assert_(s.dt == 0.1)
+
+        s = 3 * s_discrete
+        assert_(s.dt == 0.1)
+
+        s = -s_discrete
+        assert_(s.dt == 0.1)
+
+class TestTransferFunction:
     def test_initialization(self):
         # Check that all initializations work
-        s = TransferFunction(1, 1)
-        s = TransferFunction([1], [2])
-        s = TransferFunction(np.array([1]), np.array([2]))
+        TransferFunction(1, 1)
+        TransferFunction([1], [2])
+        TransferFunction(np.array([1]), np.array([2]))
 
     def test_conversion(self):
         # Check the conversion functions
@@ -933,12 +948,12 @@ class TestTransferFunction(object):
         assert_equal(s.zeros, [0])
 
 
-class TestZerosPolesGain(object):
+class TestZerosPolesGain:
     def test_initialization(self):
         # Check that all initializations work
-        s = ZerosPolesGain(1, 1, 1)
-        s = ZerosPolesGain([1], [2], 1)
-        s = ZerosPolesGain(np.array([1]), np.array([2]), 1)
+        ZerosPolesGain(1, 1, 1)
+        ZerosPolesGain([1], [2], 1)
+        ZerosPolesGain(np.array([1]), np.array([2]), 1)
 
     def test_conversion(self):
         #Check the conversion functions
@@ -952,7 +967,7 @@ class TestZerosPolesGain(object):
         assert_(s.to_zpk() is not s)
 
 
-class Test_abcd_normalize(object):
+class Test_abcd_normalize:
     def setup_method(self):
         self.A = np.array([[1.0, 2.0], [3.0, 4.0]])
         self.B = np.array([[-1.0], [5.0]])
@@ -1086,7 +1101,7 @@ class Test_abcd_normalize(object):
         assert_raises(ValueError, abcd_normalize, A=self.A, B=self.B)
 
 
-class Test_bode(object):
+class Test_bode:
 
     def test_01(self):
         # Test bode() magnitude calculation (manual sanity check).
@@ -1186,7 +1201,7 @@ class Test_bode(object):
         assert_almost_equal(mag, expected_magnitude)
 
 
-class Test_freqresp(object):
+class Test_freqresp:
 
     def test_output_manual(self):
         # Test freqresp() output calculation (manual sanity check).
@@ -1260,4 +1275,3 @@ class Test_freqresp(object):
         expected = 1 / (s + 1)**4
         assert_almost_equal(H.real, expected.real)
         assert_almost_equal(H.imag, expected.imag)
-
